/* Sample application to demonstrat FT800 primitives, widgets and customized screen shots */

#include "FT_Platform.h"
#include "FT_Buffer.h"
#include "FT_MQueue.h"
#include "FT_AVIParser.h"
#include "FT_App_PlayVideo.h"
#if defined(FT900_PLATFORM)
#include "ff.h"
#endif
#define SAMAPP_DELAY_BTW_APIS (1000)
#define SAMAPP_ENABLE_DELAY() Ft_Gpu_Hal_Sleep(SAMAPP_DELAY_BTW_APIS)
#define SAMAPP_ENABLE_DELAY_VALUE(x) Ft_Gpu_Hal_Sleep(x)
#define F16(s)        ((ft_int32_t)((s) * 65536))
#define WRITE2CMD(a) Ft_Gpu_Hal_WrCmdBuf(phost,a,sizeof(a))

#define SQ(v) (v*v)
#define NOTOUCH		-32768

#if defined(FT900_PLATFORM)
#include <ft900_sys.h>
#include <ft900_delay.h>
#include <registers/ft900_registers.h>
#endif

/* Global variables for display resolution to support various display panels */
/* Default is WQVGA - 480x272 */
ft_int16_t FT_DispWidth = 480;
ft_int16_t FT_DispHeight = 272;
ft_int16_t FT_DispHCycle =  548;
ft_int16_t FT_DispHOffset = 43;
ft_int16_t FT_DispHSync0 = 0;
ft_int16_t FT_DispHSync1 = 41;
ft_int16_t FT_DispVCycle = 292;
ft_int16_t FT_DispVOffset = 12;
ft_int16_t FT_DispVSync0 = 0;
ft_int16_t FT_DispVSync1 = 10;
ft_uint8_t FT_DispPCLK = 5;
ft_char8_t FT_DispSwizzle = 0;
ft_char8_t FT_DispPCLKPol = 1;
ft_char8_t FT_DispCSpread = 1;
ft_char8_t FT_DispDither = 1;

/* Global used for buffer optimization */



#ifdef FT900_PLATFORM
FATFS FatFs;
FIL 			 CurFile;
FRESULT          fResult;
SDHOST_STATUS    SDHostStatus;

#endif

ft_void_t ft_timer_interrupt1();

Ft_Gpu_Hal_Context_t host,*phost;

ft_uint32_t Ft_CmdBuffer_Index;
ft_uint32_t Ft_DlBuffer_Index;

#ifdef BUFFER_OPTIMIZATION
ft_uint8_t  Ft_DlBuffer[FT_DL_SIZE];
ft_uint8_t  Ft_CmdBuffer[FT_CMD_FIFO_SIZE];
#endif
/* Boot up for FT800 followed by graphics primitive sample cases */
/* Initial boot up DL - make the back ground green color */
const ft_uint8_t FT_DLCODE_BOOTUP[12] = 
{
    0,0,0,2,//GPU instruction CLEAR_COLOR_RGB
    7,0,0,38, //GPU instruction CLEAR

    0,0,0,0,  //GPU instruction DISPLAY
};
ft_void_t Ft_App_WrCoCmd_Buffer(Ft_Gpu_Hal_Context_t *phost,ft_uint32_t cmd)
{
#ifdef  BUFFER_OPTIMIZATION
   /* Copy the command instruction into buffer */
   ft_uint32_t *pBuffcmd;
   pBuffcmd =(ft_uint32_t*)&Ft_CmdBuffer[Ft_CmdBuffer_Index];
   *pBuffcmd = cmd;
#endif
#ifdef ARDUINO_PLATFORM
   Ft_Gpu_Hal_WrCmd32(phost,cmd);
#endif
#ifdef FT900_PLATFORM
   Ft_Gpu_Hal_WrCmd32(phost,cmd);
#endif
   /* Increment the command index */
   Ft_CmdBuffer_Index += FT_CMD_SIZE;  
}

ft_void_t Ft_App_WrDlCmd_Buffer(Ft_Gpu_Hal_Context_t *phost,ft_uint32_t cmd)
{
#ifdef BUFFER_OPTIMIZATION  
   /* Copy the command instruction into buffer */
   ft_uint32_t *pBuffcmd;
   pBuffcmd =(ft_uint32_t*)&Ft_DlBuffer[Ft_DlBuffer_Index];
   *pBuffcmd = cmd;
#endif

#ifdef ARDUINO_PLATFORM
   Ft_Gpu_Hal_Wr32(phost,(RAM_DL+Ft_DlBuffer_Index),cmd);
#endif
#ifdef FT900_PLATFORM
   Ft_Gpu_Hal_Wr32(phost,(RAM_DL+Ft_DlBuffer_Index),cmd);
#endif
   /* Increment the command index */
   Ft_DlBuffer_Index += FT_CMD_SIZE;  
}

ft_void_t Ft_App_WrCoStr_Buffer(Ft_Gpu_Hal_Context_t *phost,const ft_char8_t *s)
{
#ifdef  BUFFER_OPTIMIZATION  
  ft_uint16_t length = 0;
  length = strlen(s) + 1;//last for the null termination
  
  strcpy(&Ft_CmdBuffer[Ft_CmdBuffer_Index],s);  

  /* increment the length and align it by 4 bytes */
  Ft_CmdBuffer_Index += ((length + 3) & ~3);  
#endif  
}

ft_void_t Ft_App_Flush_DL_Buffer(Ft_Gpu_Hal_Context_t *phost)
{
#ifdef  BUFFER_OPTIMIZATION    
   if (Ft_DlBuffer_Index> 0)
     Ft_Gpu_Hal_WrMem(phost,RAM_DL,Ft_DlBuffer,Ft_DlBuffer_Index);
#endif     
   Ft_DlBuffer_Index = 0;
   
}

ft_void_t Ft_App_Flush_Co_Buffer(Ft_Gpu_Hal_Context_t *phost)
{
#ifdef  BUFFER_OPTIMIZATION    
   if (Ft_CmdBuffer_Index > 0)
     Ft_Gpu_Hal_WrCmdBuf(phost,Ft_CmdBuffer,Ft_CmdBuffer_Index);
#endif     
   Ft_CmdBuffer_Index = 0;
}

/* API to check the status of previous DLSWAP and perform DLSWAP of new DL */
/* Check for the status of previous DLSWAP and if still not done wait for few ms and check again */
ft_void_t SAMAPP_GPU_DLSwap(ft_uint8_t DL_Swap_Type)
{
	ft_uint8_t Swap_Type = DLSWAP_FRAME,Swap_Done = DLSWAP_FRAME;

	if(DL_Swap_Type == DLSWAP_LINE)
	{
		Swap_Type = DLSWAP_LINE;
	}

	/* Perform a new DL swap */
	Ft_Gpu_Hal_Wr8(phost,REG_DLSWAP,Swap_Type);

	/* Wait till the swap is done */
	while(Swap_Done)
	{
		Swap_Done = Ft_Gpu_Hal_Rd8(phost,REG_DLSWAP);

		if(DLSWAP_DONE != Swap_Done)
		{
			Ft_Gpu_Hal_Sleep(10);//wait for 10ms
		}
	}	
}


ft_void_t Ft_BootupConfig()
{
	Ft_Gpu_Hal_Powercycle(phost,FT_TRUE);

		/* Access address 0 to wake up the FT800 */
		Ft_Gpu_HostCommand(phost,FT_GPU_ACTIVE_M);
		Ft_Gpu_Hal_Sleep(20);

		/* Set the clk to external clock */
#if (!defined(ME800A_HV35R) && !defined(ME810A_HV35R))
		Ft_Gpu_HostCommand(phost,FT_GPU_EXTERNAL_OSC);
		Ft_Gpu_Hal_Sleep(10);
#endif

		{
			ft_uint8_t chipid;
			//Read Register ID to check if FT800 is ready.
			chipid = Ft_Gpu_Hal_Rd8(phost, REG_ID);
			while(chipid != 0x7C)
			{
				chipid = Ft_Gpu_Hal_Rd8(phost, REG_ID);
				ft_delay(100);
			}
	#if defined(MSVC_PLATFORM) || defined (FT900_PLATFORM)
			printf("VC1 register ID after wake up %x\n",chipid);
	#endif
	}
	/* Configuration of LCD display */
#ifdef DISPLAY_RESOLUTION_QVGA
	/* Values specific to QVGA LCD display */
	FT_DispWidth = 320;
	FT_DispHeight = 240;
	FT_DispHCycle =  408;
	FT_DispHOffset = 70;
	FT_DispHSync0 = 0;
	FT_DispHSync1 = 10;
	FT_DispVCycle = 263;
	FT_DispVOffset = 13;
	FT_DispVSync0 = 0;
	FT_DispVSync1 = 2;
	FT_DispPCLK = 8;
	FT_DispSwizzle = 2;
	FT_DispPCLKPol = 0;
	FT_DispCSpread = 1;
	FT_DispDither = 1;

#endif
#ifdef DISPLAY_RESOLUTION_WVGA
	/* Values specific to QVGA LCD display */
	FT_DispWidth = 800;
	FT_DispHeight = 480;
	FT_DispHCycle =  928;
	FT_DispHOffset = 88;
	FT_DispHSync0 = 0;
	FT_DispHSync1 = 48;
	FT_DispVCycle = 525;
	FT_DispVOffset = 32;
	FT_DispVSync0 = 0;
	FT_DispVSync1 = 3;
	FT_DispPCLK = 2;
	FT_DispSwizzle = 0;
	FT_DispPCLKPol = 1;
	FT_DispCSpread = 0;
	FT_DispDither = 1;
#endif
#ifdef DISPLAY_RESOLUTION_HVGA_PORTRAIT
	/* Values specific to HVGA LCD display */

	FT_DispWidth = 320;
	FT_DispHeight = 480;
	FT_DispHCycle =  400;
	FT_DispHOffset = 40;
	FT_DispHSync0 = 0;
	FT_DispHSync1 = 10;
	FT_DispVCycle = 500;
	FT_DispVOffset = 10;
	FT_DispVSync0 = 0;
	FT_DispVSync1 = 5;
	FT_DispPCLK = 4;
	FT_DispSwizzle = 2;
	FT_DispPCLKPol = 1;
	FT_DispCSpread = 1;
	FT_DispDither = 1;

#ifdef ME810A_HV35R
	FT_DispPCLK = 5;
#endif

#endif

#if (defined(ME800A_HV35R) || defined(ME810A_HV35R))
	/* After recognizing the type of chip, perform the trimming if necessary */
    Ft_Gpu_ClockTrimming(phost,LOW_FREQ_BOUND);
#endif

	Ft_Gpu_Hal_Wr16(phost, REG_HCYCLE, FT_DispHCycle);
	Ft_Gpu_Hal_Wr16(phost, REG_HOFFSET, FT_DispHOffset);
	Ft_Gpu_Hal_Wr16(phost, REG_HSYNC0, FT_DispHSync0);
	Ft_Gpu_Hal_Wr16(phost, REG_HSYNC1, FT_DispHSync1);
	Ft_Gpu_Hal_Wr16(phost, REG_VCYCLE, FT_DispVCycle);
	Ft_Gpu_Hal_Wr16(phost, REG_VOFFSET, FT_DispVOffset);
	Ft_Gpu_Hal_Wr16(phost, REG_VSYNC0, FT_DispVSync0);
	Ft_Gpu_Hal_Wr16(phost, REG_VSYNC1, FT_DispVSync1);
	Ft_Gpu_Hal_Wr8(phost, REG_SWIZZLE, FT_DispSwizzle);
	Ft_Gpu_Hal_Wr8(phost, REG_PCLK_POL, FT_DispPCLKPol);
	Ft_Gpu_Hal_Wr16(phost, REG_HSIZE, FT_DispWidth);
	Ft_Gpu_Hal_Wr16(phost, REG_VSIZE, FT_DispHeight);
	Ft_Gpu_Hal_Wr16(phost, REG_CSPREAD, FT_DispCSpread);
	Ft_Gpu_Hal_Wr16(phost, REG_DITHER, FT_DispDither);

#if (defined(FT_800_ENABLE) || defined(FT_810_ENABLE) ||defined(FT_812_ENABLE))
    /* Touch configuration - configure the resistance value to 1200 - this value is specific to customer requirement and derived by experiment */
    Ft_Gpu_Hal_Wr16(phost, REG_TOUCH_RZTHRESH,RESISTANCE_THRESHOLD);
#endif
    Ft_Gpu_Hal_Wr8(phost, REG_GPIO_DIR,0xff);
    Ft_Gpu_Hal_Wr8(phost, REG_GPIO,0xff);


    /*It is optional to clear the screen here*/
    Ft_Gpu_Hal_WrMem(phost, RAM_DL,(ft_uint8_t *)FT_DLCODE_BOOTUP,sizeof(FT_DLCODE_BOOTUP));
    Ft_Gpu_Hal_Wr8(phost, REG_DLSWAP,DLSWAP_FRAME);


    Ft_Gpu_Hal_Wr8(phost, REG_PCLK,FT_DispPCLK);//after this display is visible on the LCD


#ifdef ENABLE_ILI9488_HVGA_PORTRAIT
	/* to cross check reset pin */
	Ft_Gpu_Hal_Wr8(phost, REG_GPIO,0xff);
	ft_delay(120);
	Ft_Gpu_Hal_Wr8(phost, REG_GPIO,0x7f);
	ft_delay(120);
	Ft_Gpu_Hal_Wr8(phost, REG_GPIO,0xff);

	ILI9488_Bootup();

	/* Reconfigure the SPI */
#ifdef FT900_PLATFORM
	printf("after ILI9488 bootup \n");
	//spi
	// Initialize SPIM HW
	sys_enable(sys_device_spi_master);
	gpio_function(27, pad_spim_sck); /* GPIO27 to SPIM_CLK */
	gpio_function(28, pad_spim_ss0); /* GPIO28 as CS */
	gpio_function(29, pad_spim_mosi); /* GPIO29 to SPIM_MOSI */
	gpio_function(30, pad_spim_miso); /* GPIO30 to SPIM_MISO */

	gpio_write(28, 1);
	spi_init(SPIM, spi_dir_master, spi_mode_0, 4);
#endif

#endif



	/* make the spi to quad mode - addition 2 bytes for silicon */
#ifdef FT_81X_ENABLE
	/* api to set quad and numbe of dummy bytes */
#ifdef ENABLE_SPI_QUAD
	Ft_Gpu_Hal_SetSPI(phost,FT_GPU_SPI_QUAD_CHANNEL,FT_GPU_SPI_TWODUMMY);
#elif ENABLE_SPI_DUAL
	Ft_Gpu_Hal_SetSPI(phost,FT_GPU_SPI_DUAL_CHANNEL,FT_GPU_SPI_TWODUMMY);
#else
	Ft_Gpu_Hal_SetSPI(phost,FT_GPU_SPI_SINGLE_CHANNEL,FT_GPU_SPI_ONEDUMMY);
#endif

#endif

#ifdef FT900_PLATFORM
    /* Change clock frequency to 25mhz */
	spi_init(SPIM, spi_dir_master, spi_mode_0, 4);

#if (defined(ENABLE_SPI_QUAD))
    /* Initialize IO2 and IO3 pad/pin for dual and quad settings */
    gpio_function(31, pad_spim_io2);
    gpio_function(32, pad_spim_io3);
    gpio_write(31, 1);
    gpio_write(32, 1);
#endif
	/* Enable FIFO of QSPI */
	spi_option(SPIM,spi_option_fifo_size,64);
	spi_option(SPIM,spi_option_fifo,1);
	spi_option(SPIM,spi_option_fifo_receive_trigger,1);
#endif

#ifdef ENABLE_SPI_QUAD
#ifdef FT900_PLATFORM
	spi_option(SPIM,spi_option_bus_width,4);
#endif
#elif ENABLE_SPI_DUAL	
#ifdef FT900_PLATFORM
	spi_option(SPIM,spi_option_bus_width,2);
#endif
#else
#ifdef FT900_PLATFORM
	spi_option(SPIM,spi_option_bus_width,1);
#endif

	
#endif


phost->ft_cmd_fifo_wp = Ft_Gpu_Hal_Rd16(phost,REG_CMD_WRITE);

}


/*****************************************************************************/
/* Example code to display few points at various offsets with various colors */

static ft_int32_t ox;
static ft_uint8_t noofch=0;

static ft_uint8_t home_star_icon[] = {0x78,0x9C,0xE5,0x94,0xBF,0x4E,0xC2,0x40,0x1C,0xC7,0x7F,0x2D,0x04,0x8B,0x20,0x45,0x76,0x14,0x67,0xA3,0xF1,0x0D,0x64,0x75,0xD2,0xD5,0x09,0x27,0x17,0x13,0xE1,0x0D,0xE4,0x0D,0x78,0x04,0x98,0x5D,0x30,0x26,0x0E,0x4A,0xA2,0x3E,0x82,0x0E,0x8E,0x82,0xC1,0x38,0x62,0x51,0x0C,0x0A,0x42,0x7F,0xDE,0xB5,0x77,0xB4,0x77,0x17,0x28,0x21,0x26,0x46,0xFD,0x26,0xCD,0xE5,0xD3,0x7C,0xFB,0xBB,0xFB,0xFD,0xB9,0x02,0xCC,0xA4,0xE8,0x99,0x80,0x61,0xC4,0x8A,0x9F,0xCB,0x6F,0x31,0x3B,0xE3,0x61,0x7A,0x98,0x84,0x7C,0x37,0xF6,0xFC,0xC8,0xDD,0x45,0x00,0xDD,0xBA,0xC4,0x77,0xE6,0xEE,0x40,0xEC,0x0E,0xE6,0x91,0xF1,0xD2,0x00,0x42,0x34,0x5E,0xCE,0xE5,0x08,0x16,0xA0,0x84,0x68,0x67,0xB4,0x86,0xC3,0xD5,0x26,0x2C,0x20,0x51,0x17,0xA2,0xB8,0x03,0xB0,0xFE,0x49,0xDD,0x54,0x15,0xD8,0xEE,0x73,0x37,0x95,0x9D,0xD4,0x1A,0xB7,0xA5,0x26,0xC4,0x91,0xA9,0x0B,0x06,0xEE,0x72,0xB7,0xFB,0xC5,0x16,0x80,0xE9,0xF1,0x07,0x8D,0x3F,0x15,0x5F,0x1C,0x0B,0xFC,0x0A,0x90,0xF0,0xF3,0x09,0xA9,0x90,0xC4,0xC6,0x37,0xB0,0x93,0xBF,0xE1,0x71,0xDB,0xA9,0xD7,0x41,0xAD,0x46,0xEA,0x19,0xA9,0xD5,0xCE,0x93,0xB3,0x35,0x73,0x0A,0x69,0x59,0x91,0xC3,0x0F,0x22,0x1B,0x1D,0x91,0x13,0x3D,0x91,0x73,0x43,0xF1,0x6C,0x55,0xDA,0x3A,0x4F,0xBA,0x25,0xCE,0x4F,0x04,0xF1,0xC5,0xCF,0x71,0xDA,0x3C,0xD7,0xB9,0xB2,0x48,0xB4,0x89,0x38,0x20,0x4B,0x2A,0x95,0x0C,0xD5,0xEF,0x5B,0xAD,0x96,0x45,0x8A,0x41,0x96,0x7A,0x1F,0x60,0x0D,0x7D,0x22,0x75,0x82,0x2B,0x0F,0xFB,0xCE,0x51,0x3D,0x2E,0x3A,0x21,0xF3,0x1C,0xD9,0x38,0x86,0x2C,0xC6,0x05,0xB6,0x7B,0x9A,0x8F,0x0F,0x97,0x1B,0x72,0x6F,0x1C,0xEB,0xAE,0xFF,0xDA,0x97,0x0D,0xBA,0x43,0x32,0xCA,0x66,0x34,0x3D,0x54,0xCB,0x24,0x9B,0x43,0xF2,0x70,0x3E,0x42,0xBB,0xA0,0x95,0x11,0x37,0x46,0xE1,0x4F,0x49,0xC5,0x1B,0xFC,0x3C,0x3A,0x3E,0xD1,0x65,0x0E,0x6F,0x58,0xF8,0x9E,0x5B,0xDB,0x55,0xB6,0x41,0x34,0xCB,0xBE,0xDB,0x87,0x5F,0xA9,0xD1,0x85,0x6B,0xB3,0x17,0x9C,0x61,0x0C,0x9B,0xA2,0x5D,0x61,0x10,0xED,0x2A,0x9B,0xA2,0x5D,0x61,0x10,0xED,0x2A,0x9B,0xA2,0x5D,0x61,0x10,0xED,0x2A,0x9B,0xED,0xC9,0xFC,0xDF,0x14,0x54,0x8F,0x80,0x7A,0x06,0xF5,0x23,0xA0,0x9F,0x41,0xF3,0x10,0x30,0x4F,0x41,0xF3,0x18,0x30,0xCF,0xCA,0xFC,0xFF,0x35,0xC9,0x79,0xC9,0x89,0xFA,0x33,0xD7,0x1D,0xF6,0x5E,0x84,0x5C,0x56,0x6E,0xA7,0xDA,0x1E,0xF9,0xFA,0xAB,0xF5,0x97,0xFF,0x2F,0xED,0x89,0x7E,0x29,0x9E,0xB4,0x9F,0x74,0x1E,0x69,0xDA,0xA4,0x9F,0x81,0x94,0xEF,0x4F,0xF6,0xF9,0x0B,0xF4,0x65,0x51,0x08};

static void polarxy(ft_int32_t r, ft_uint16_t th, ft_int32_t *x, ft_int32_t *y)
{
  *x = (16 * (FT_DispWidth/(2*noofch)) + (((long)r * qsin(th)) >> 11) + 16 * ox);
  *y = (16 * 300 - (((long)r * qcos(th)) >> 11));
}


void vertex(ft_int32_t x, ft_int32_t y)
{
  Ft_App_WrCoCmd_Buffer(phost,VERTEX2F(x,y));
}


static void polar(ft_int32_t r, ft_uint16_t th)
{
  ft_int32_t x, y;
  polarxy(r, th, &x, &y);
  vertex(x, y);
}

ft_uint16_t da(ft_int32_t i)
{
  return (i - 45) * 32768L / 360;
}

static void cs(ft_uint8_t i)
{
  switch (i)
  {
    case  0:  Ft_App_WrCoCmd_Buffer(phost,COLOR_RGB(200,255,200)); break;
    case 60: Ft_App_WrCoCmd_Buffer(phost,COLOR_RGB(255,255,0)); break;
    case 80: Ft_App_WrCoCmd_Buffer(phost,COLOR_RGB(255,0,0)); break;
  }
}

ft_void_t home_setup()
{
  Ft_Gpu_Hal_WrCmd32(phost,CMD_INFLATE);
  Ft_Gpu_Hal_WrCmd32(phost,250*1024L);
  Ft_Gpu_Hal_WrCmdBuf(phost,home_star_icon,sizeof(home_star_icon));
  
  Ft_Gpu_CoCmd_Dlstart(phost);        // start
  Ft_App_WrCoCmd_Buffer(phost,CLEAR(1,1,1));
  Ft_App_WrCoCmd_Buffer(phost,COLOR_RGB(255, 255, 255));
  Ft_App_WrCoCmd_Buffer(phost,BITMAP_HANDLE(13));    // handle for background stars
  Ft_App_WrCoCmd_Buffer(phost,BITMAP_SOURCE(250*1024L));      // Starting address in gram
  Ft_App_WrCoCmd_Buffer(phost,BITMAP_LAYOUT(L4, 16, 32));  // format 
  Ft_App_WrCoCmd_Buffer(phost,BITMAP_SIZE(NEAREST, REPEAT, REPEAT, 512, 512  ));
  Ft_App_WrCoCmd_Buffer(phost,BITMAP_HANDLE(14));    // handle for background stars
  Ft_App_WrCoCmd_Buffer(phost,BITMAP_SOURCE(250*1024L));      // Starting address in gram
  Ft_App_WrCoCmd_Buffer(phost,BITMAP_LAYOUT(L4, 16, 32));  // format 
  Ft_App_WrCoCmd_Buffer(phost,BITMAP_SIZE(NEAREST, BORDER, BORDER, 32, 32  ));
  Ft_App_WrCoCmd_Buffer(phost,DISPLAY());
  Ft_Gpu_CoCmd_Swap(phost);
  Ft_App_Flush_Co_Buffer(phost);
  Ft_Gpu_Hal_WaitCmdfifo_empty(phost);
}


static ft_uint8_t sk=0;
ft_uint8_t Read_Keys()
{
  static ft_uint8_t Read_tag=0,temp_tag=0,ret_tag=0;	
  Read_tag = Ft_Gpu_Hal_Rd8(phost,REG_TOUCH_TAG);
  ret_tag = 0;
  if(Read_tag != 0)								// Allow if the Key is released
  {
    if(temp_tag!=Read_tag)
    {
      temp_tag = Read_tag;	
      sk = Read_tag;										// Load the Read tag to temp variable	
    }  
  }
  else
  {
    if(temp_tag!=0)
    {
      ret_tag = temp_tag;
    }  
    sk = 0;
  }
  return ret_tag;
}



char *info[] = { "FT800 Play Video Application",

				"APP to demonstrate video playback,",
				"using mpjeg decode feature",
				"& fifo buffer."
                       };

void Ft_Play_Sound(ft_uint8_t sound,ft_uint8_t vol,ft_uint8_t midi)
{
  ft_uint16_t val = (midi << 8) | sound; 
  Ft_Gpu_Hal_Wr16(phost,REG_SOUND,val);
  Ft_Gpu_Hal_Wr8(phost,REG_PLAY,1); 
}
ft_void_t Info()
{
  ft_uint16_t dloffset = 0,z;
  Ft_CmdBuffer_Index = 0;
  Ft_Gpu_CoCmd_Dlstart(phost); 
  Ft_App_WrCoCmd_Buffer(phost,CLEAR(1,1,1));
  Ft_App_WrCoCmd_Buffer(phost,COLOR_RGB(255,255,255));
  Ft_Gpu_CoCmd_Text(phost,FT_DispWidth/2,FT_DispHeight/2,26,OPT_CENTERX|OPT_CENTERY,"Please tap on a dot");
  Ft_Gpu_CoCmd_Calibrate(phost,0);
  Ft_App_Flush_Co_Buffer(phost);
  Ft_Gpu_Hal_WaitCmdfifo_empty(phost);
  
  //commenting the logo functionality due to not support on ft81x yet
  Ft_Gpu_CoCmd_Logo(phost);
  Ft_App_Flush_Co_Buffer(phost);

  Ft_Gpu_Hal_WaitLogo_Finish(phost);
  dloffset = Ft_Gpu_Hal_Rd16(phost,REG_CMD_DL);
  dloffset -= 4;

#ifdef FT_81X_ENABLE
  dloffset -= 2*4;//remove two more instructions in case of 81x
#endif


  Ft_Gpu_Hal_WrCmd32(phost,CMD_MEMCPY);
  Ft_Gpu_Hal_WrCmd32(phost,100000L);
  Ft_Gpu_Hal_WrCmd32(phost,RAM_DL);
  Ft_Gpu_Hal_WrCmd32(phost,dloffset);

  do
  {
    Ft_Gpu_CoCmd_Dlstart(phost);   
    Ft_Gpu_CoCmd_Append(phost,100000L,dloffset);
    Ft_App_WrCoCmd_Buffer(phost,BITMAP_TRANSFORM_A(256));
    Ft_App_WrCoCmd_Buffer(phost,BITMAP_TRANSFORM_A(256));
    Ft_App_WrCoCmd_Buffer(phost,BITMAP_TRANSFORM_B(0));
    Ft_App_WrCoCmd_Buffer(phost,BITMAP_TRANSFORM_C(0));
    Ft_App_WrCoCmd_Buffer(phost,BITMAP_TRANSFORM_D(0));
    Ft_App_WrCoCmd_Buffer(phost,BITMAP_TRANSFORM_E(256));
    Ft_App_WrCoCmd_Buffer(phost,BITMAP_TRANSFORM_F(0));  
    Ft_App_WrCoCmd_Buffer(phost,SAVE_CONTEXT());	
    Ft_App_WrCoCmd_Buffer(phost,COLOR_RGB(219,180,150));
    Ft_App_WrCoCmd_Buffer(phost,COLOR_A(220));
    Ft_App_WrCoCmd_Buffer(phost,BEGIN(EDGE_STRIP_A));
    Ft_App_WrCoCmd_Buffer(phost,VERTEX2F(0,FT_DispHeight*16));
    Ft_App_WrCoCmd_Buffer(phost,VERTEX2F(FT_DispWidth*16,FT_DispHeight*16));
    Ft_App_WrCoCmd_Buffer(phost,COLOR_A(255));
    Ft_App_WrCoCmd_Buffer(phost,RESTORE_CONTEXT());	
    Ft_App_WrCoCmd_Buffer(phost,COLOR_RGB(0,0,0));
   // INFORMATION 
    Ft_Gpu_CoCmd_Text(phost,FT_DispWidth/2,20,28,OPT_CENTERX|OPT_CENTERY,info[0]);
    Ft_Gpu_CoCmd_Text(phost,FT_DispWidth/2,60,26,OPT_CENTERX|OPT_CENTERY,info[1]);
    Ft_Gpu_CoCmd_Text(phost,FT_DispWidth/2,90,26,OPT_CENTERX|OPT_CENTERY,info[2]);  
    Ft_Gpu_CoCmd_Text(phost,FT_DispWidth/2,120,26,OPT_CENTERX|OPT_CENTERY,info[3]);  
    Ft_Gpu_CoCmd_Text(phost,FT_DispWidth/2,FT_DispHeight-30,26,OPT_CENTERX|OPT_CENTERY,"Click to play");
    if(sk!='P')
    Ft_App_WrCoCmd_Buffer(phost,COLOR_RGB(255,255,255));
    else
    Ft_App_WrCoCmd_Buffer(phost,COLOR_RGB(100,100,100));
    Ft_App_WrCoCmd_Buffer(phost,BEGIN(FTPOINTS));   
    Ft_App_WrCoCmd_Buffer(phost,POINT_SIZE(20*16));
    Ft_App_WrCoCmd_Buffer(phost,TAG('P'));
    Ft_App_WrCoCmd_Buffer(phost,VERTEX2F((FT_DispWidth/2)*16,(FT_DispHeight-60)*16));
    Ft_App_WrCoCmd_Buffer(phost,COLOR_RGB(180,35,35));
    Ft_App_WrCoCmd_Buffer(phost,BEGIN(BITMAPS));
    Ft_App_WrCoCmd_Buffer(phost,VERTEX2II((FT_DispWidth/2)-14,(FT_DispHeight-75),14,4));
    Ft_App_WrCoCmd_Buffer(phost,DISPLAY());
    Ft_Gpu_CoCmd_Swap(phost);
    Ft_App_Flush_Co_Buffer(phost);
    Ft_Gpu_Hal_WaitCmdfifo_empty(phost);
  }while(Read_Keys()!='P');
   Ft_Play_Sound(0x50,255,0xc0);
}


#ifdef FT900_PLATFORM

ft_void_t FT900_Config()
{
	sys_enable(sys_device_uart0);
    gpio_function(48, pad_uart0_txd); /* UART0 TXD */
    gpio_function(49, pad_uart0_rxd); /* UART0 RXD */
    uart_open(UART0,                    /* Device */
              1,                        /* Prescaler = 1 */
              UART_DIVIDER_115200_BAUD,  /* Divider = 1302 */
              uart_data_bits_8,         /* No. Data Bits */
              uart_parity_none,         /* Parity */
              uart_stop_bits_1);        /* No. Stop Bits */

    /* Print out a welcome message... */
    uart_puts(UART0,

        "(C) Copyright 2014-2015, Future Technology Devices International Ltd. \r\n"
        "--------------------------------------------------------------------- \r\n"
        "Welcome to Video Player Example ... \r\n"
        "\r\n"
        "--------------------------------------------------------------------- \r\n"
        );

#ifdef ENABLE_ILI9488_HVGA_PORTRAIT
/* asign all the respective pins to gpio and set them to default values */
gpio_function(34, pad_gpio34);
gpio_dir(34, pad_dir_output);
gpio_write(34,1);

gpio_function(27, pad_gpio27);
gpio_dir(27, pad_dir_output);
gpio_write(27,1);

gpio_function(29, pad_gpio29);
gpio_dir(29, pad_dir_output);
gpio_write(29,1);

gpio_function(33, pad_gpio33);
gpio_dir(33, pad_dir_output);
gpio_write(33,1);


gpio_function(30, pad_gpio30);
gpio_dir(30, pad_dir_output);
gpio_write(30,1);

gpio_function(28, pad_gpio28);
gpio_dir(28, pad_dir_output);
gpio_write(28,1);


	gpio_function(43, pad_gpio43);
	gpio_dir(43, pad_dir_output);
gpio_write(43,1);
gpio_write(34,1);
gpio_write(28,1);
gpio_write(43,1);
gpio_write(33,1);
gpio_write(33,1);

#endif
/* useful for sd card */
sys_enable(sys_device_sd_card);
	sdhost_init();
	#define GPIO_SD_CLK  (19)
	#define GPIO_SD_CMD  (20)
	#define GPIO_SD_DAT3 (21)
	#define GPIO_SD_DAT2 (22)
	#define GPIO_SD_DAT1 (23)
	#define GPIO_SD_DAT0 (24)
	#define GPIO_SD_CD   (25)
	#define GPIO_SD_WP   (26)
	gpio_function(GPIO_SD_CLK, pad_sd_clk); gpio_pull(GPIO_SD_CLK, pad_pull_none);//pad_pull_none
	gpio_function(GPIO_SD_CMD, pad_sd_cmd); gpio_pull(GPIO_SD_CMD, pad_pull_pullup);
	gpio_function(GPIO_SD_DAT3, pad_sd_data3); gpio_pull(GPIO_SD_DAT3, pad_pull_pullup);
    gpio_function(GPIO_SD_DAT2, pad_sd_data2); gpio_pull(GPIO_SD_DAT2, pad_pull_pullup);
	gpio_function(GPIO_SD_DAT1, pad_sd_data1); gpio_pull(GPIO_SD_DAT1, pad_pull_pullup);
	gpio_function(GPIO_SD_DAT0, pad_sd_data0); gpio_pull(GPIO_SD_DAT0, pad_pull_pullup);
	gpio_function(GPIO_SD_CD, pad_sd_cd); gpio_pull(GPIO_SD_CD, pad_pull_pullup);
	gpio_function(GPIO_SD_WP, pad_sd_wp); gpio_pull(GPIO_SD_WP, pad_pull_pullup);
	SDHOST_STATUS sd_status;
	if(sdhost_card_detect() == SDHOST_CARD_INSERTED)
	{
		printf("\n sd_status=%s",sdhost_card_detect());
	}
	if (sdhost_card_detect() != SDHOST_CARD_INSERTED)
	{
		printf("Please Insert SD Card\r\n");
	}
	else
	{
		printf("\n SD Card inserted!");
	}
	if (f_mount(&FatFs, "", 0) != FR_OK)
	{
		printf("\n Mounted failed.");;
	}
	printf("\n Mounted succesfully.");
/* useful for timer */
//ft_millis_init();
//interrupt_enable_globally();
//printf("ft900 config done \n");
}


/* Globals for polling implementation */
ft_uint32_t ft_us_curr = 0,ft_us_prev = 0;

/* Globals for interrupt implementation */
volatile ft_uint32_t FTPayer_TMicroS = 0,FTPlayer_millis_curr = 0,FTPlayer_millis_prev = 0;



/* Time for each interrupt would be (1000/(frequency/(prescalar*timervalue))). to get 30fps or 33.333ms, 1000/(100000000/(30*1000*100))*/
ft_void_t ft_timer_interrupt1_init(ft_int32_t TPrescalar,ft_int32_t TValue)
{
	uint16_t value;
#ifdef FT900_PLATFORM
	FTPlayer_millis_curr = 0;
	FTPlayer_millis_prev = 0;
	FTPayer_TMicroS = 0;
	ft_us_curr = 0;
	ft_us_prev = 0;
		 timer_init(timer_select_a,              /* Device */
				 TValue,        /* Initial Value */
		timer_direction_up,        /* Count Direction */
		timer_prescaler_select_on,   /* Prescaler Select */
		timer_mode_continuous);
		//	ft900_timer_set_interrupt(2, ft_timer_interrupt1);
		// interrupt_enable_globally();

		//	timer_prescaler(TPrescalar);
		 timer_prescaler(TPrescalar);
		 printf("\n tvalue =%d %d",TValue,TPrescalar);
		 interrupt_attach(interrupt_timers, 17, ft_timer_interrupt1);
		 timer_enable_interrupt(timer_select_a);

		 interrupt_enable_globally();
		 timer_start(timer_select_a);

#endif
}

/* Need to ensure that below api is called at least once in 6.5 seconds duration for FT900 platform as this module doesnt use timer for context update */
/* global counter to loopback after ~49.71 days */
ft_uint32_t ft_timer_interrupt1_get()
{
#if defined(ARDUINO_PLATFORM) ||defined(MSVC_PLATFORM) || defined(MSVC_FT800EMU)
	return millis();
#endif
#ifdef FT900_PLATFORM

	/* Interrupt implementation */
	//printf("FTPayer_TMicroS=%d",FTPayer_TMicroS);
	return (FTPayer_TMicroS);

#endif
}

ft_void_t ft_timer_interrupt1_exit()
{
#ifdef FT900_PLATFORM
	 timer_stop(timer_select_a);
#endif
}

ft_uint32_t tpprevval = 0,timercount = 0,timervalue = 0;


#endif





ft_uint8_t G_PBuff[FT_AVIPARSER_BUFFER_SIZE];
ft_uint8_t istouch()
{
  return !(Ft_Gpu_Hal_Rd16(phost,REG_TOUCH_RAW_XY) & 0x8000);
}

float linear(float p1,float p2,float t,ft_uint16_t rate)
{
	float st  = (float)t/rate;
	return p1+(st*(p2-p1));
}

ft_int16_t xoffset_array[30],yoffset_array[30],dx_pts[20],dy_pts[20],rate_cts[30],iteration_cts[30];
//ft_uint16_t point_size[30];
//ft_uint8_t color[20],alpha_array[20];

void Backgroundanimation_3()
{
	ft_int32_t i=0,rate = 1000;
	ft_uint8_t alpha  = 0,inc = 1;
	static ft_uint16_t ctime = 0,t1 = 0;
	static ft_uint8_t ptsapp = 19;
	ft_int16_t xoff = 0 ,yoff = 0;
	if(istouch())
	{
		t1 = 0; inc  = 10;
	}
	else {
		inc = linear(10,2,t1,100);
		if(t1<100) t1++;
	}
	//clear the background color

	Ft_App_WrCoCmd_Buffer(phost,SAVE_CONTEXT());
	Ft_App_WrCoCmd_Buffer(phost,CLEAR_COLOR_RGB(0,0,0));
	Ft_App_WrCoCmd_Buffer(phost,CLEAR_COLOR_A(0));
	Ft_App_WrCoCmd_Buffer(phost,CLEAR(1,1,1));
	Ft_App_WrCoCmd_Buffer(phost,COLOR_MASK(1,1,1,0));
	//draw the cmd gradient with scissors
	Ft_App_WrCoCmd_Buffer(phost,SCISSOR_SIZE(FT_DispWidth,FT_DispHeight));
	Ft_App_WrCoCmd_Buffer(phost,SCISSOR_XY(0,0));
	Ft_Gpu_CoCmd_Gradient(phost,0,0,0x708fa1,0,200,0xc4cdd2);
	Ft_App_WrCoCmd_Buffer(phost,SCISSOR_XY(0,200));
	Ft_Gpu_CoCmd_Gradient(phost,0,200,0xc4cdd2,0,400,0x4f7588);

	Ft_App_WrCoCmd_Buffer(phost,SCISSOR_XY(0,0));
	Ft_App_WrCoCmd_Buffer(phost,SCISSOR_SIZE(2048,2048));//reprogram with  default values
	Ft_Gpu_CoCmd_Text(phost,FT_DispWidth/2,FT_DispHeight - 80/2,29,OPT_CENTERX|OPT_CENTERY,"FTDI Video Player Demo");
	Ft_App_WrCoCmd_Buffer(phost,BEGIN(FTPOINTS));
	Ft_App_WrCoCmd_Buffer(phost,COLOR_MASK(1,1,1,0));
	Ft_App_WrCoCmd_Buffer(phost,COLOR_RGB(255,255,255));
	for(i=0;i<20;i++)
	{
		alpha  = linear(80,0,iteration_cts[i],rate_cts[i]);
		if(alpha<75)
		{
			Ft_App_WrCoCmd_Buffer(phost,POINT_SIZE(16*(30 + (3*i/2))));
			Ft_App_WrCoCmd_Buffer(phost,COLOR_A(alpha));
			xoff = linear(xoffset_array[i],dx_pts[i],iteration_cts[i],rate_cts[i]);
			yoff = linear(yoffset_array[i],dy_pts[i],iteration_cts[i],rate_cts[i]);
			Ft_App_WrCoCmd_Buffer(phost,VERTEX2F(xoff*16,yoff*16));
		}
	}
	Ft_App_WrCoCmd_Buffer(phost,RESTORE_CONTEXT());
	for(i = 0; i<20 ; i++)
	{
		if(iteration_cts[i]==0)
		{
			xoffset_array[i] = ft_random(FT_DispWidth);
			yoffset_array[i] = 100+ft_random(FT_DispHeight/4);
			dx_pts[i] = ft_random(FT_DispWidth);
			dy_pts[i] = ft_random(FT_DispHeight);
			rate_cts[i] = 500+ft_random(500);
		}
		if(iteration_cts[i]<rate_cts[i])iteration_cts[i]+=inc; else{ iteration_cts[i] = 0;}
	}
	return;
}

static struct {
  signed short dragprev;
  int vel;      // velocity
  long base;    // screen x coordinate, in 1/16ths pixel
  long limit;
} scroller;

static void scroller_init(ft_uint32_t limit)
{
  scroller.dragprev = -32768;
  scroller.vel = 0;      // velocity
  scroller.base = 0;     // screen x coordinate, in 1/16ths pixel
  scroller.limit = limit;
}

void scroller_run(ft_int16_t sx,ft_int32_t directxy)
{
  //signed short sx;
  static ft_uint16_t _delay = 0;

#ifdef FT_801_ENABLE
  static ft_uint32_t prev_time = 0;
  ft_uint32_t time  = GetTickCount();
  if(prev_time!=0)
  _delay += (time-prev_time);
  prev_time = time;
  if(_delay<30)
  {
	scroller.base += scroller.vel;
	scroller.base = max(0, min(scroller.base, scroller.limit));
	return;
  }
#endif

  if ((sx != -32768) && (scroller.dragprev != -32768)) {
    scroller.vel = (scroller.dragprev - sx) << 4;
  } else {
    int change = max(1, abs(scroller.vel) >> 5);
    if (scroller.vel < 0)
      scroller.vel += change;
    if (scroller.vel > 0)
      scroller.vel -= change;
  }
  scroller.dragprev = sx;
  scroller.base += scroller.vel;
  scroller.base = max(0, min(scroller.base, scroller.limit));
  _delay = 0;
}

ft_int32_t DefaultGUIScreen(Ft_Gpu_Hal_Context_t *phost)
{

	Ft_Gpu_CoCmd_Dlstart(phost);
	Ft_App_WrCoCmd_Buffer(phost,CLEAR_COLOR_RGB(0,0,0));
	Ft_App_WrCoCmd_Buffer(phost,CLEAR(1,1,1) );
	Ft_App_WrCoCmd_Buffer(phost,DISPLAY());
	Ft_Gpu_CoCmd_Swap(phost);
	Ft_App_Flush_Co_Buffer(phost);
	Ft_Gpu_Hal_WaitCmdfifo_empty(phost);

	return 0;
}

ft_int32_t VideoFrameonly(Ft_Gpu_Hal_Context_t *phost,ft_int32_t aviw,ft_int32_t avih)
{

	Ft_Gpu_CoCmd_Dlstart(phost);
	Ft_App_WrCoCmd_Buffer(phost,CLEAR_TAG(0));

	Ft_App_WrCoCmd_Buffer(phost,CLEAR_COLOR_RGB(0,0,0));
	Ft_App_WrCoCmd_Buffer(phost,CLEAR(1,1,1) );

	//decoded jpeg bitmap
	Ft_App_WrCoCmd_Buffer(phost,TAG_MASK(0));
	Ft_App_WrCoCmd_Buffer(phost,TAG(0));
	Ft_App_WrCoCmd_Buffer(phost,COLOR_RGB(255,255,255));
	Ft_App_WrCoCmd_Buffer(phost,BITMAP_HANDLE(0));
	Ft_App_WrCoCmd_Buffer(phost,BITMAP_SOURCE(0));//bitmap locaiton is 4, as 0 is used for return of video decode result
	Ft_App_WrCoCmd_Buffer(phost,BITMAP_LAYOUT(RGB565,aviw*2L,avih));
	Ft_App_WrCoCmd_Buffer(phost,BITMAP_LAYOUT_H(((aviw*2L)>>10),((avih)>>9)));
	Ft_App_WrCoCmd_Buffer(phost,BITMAP_SIZE(NEAREST,BORDER,BORDER,aviw,avih));
	Ft_App_WrCoCmd_Buffer(phost,BITMAP_SIZE_H((aviw>>9),(avih>>9)));
	Ft_App_WrCoCmd_Buffer(phost,BEGIN(BITMAPS));
	//hack for not to display video frame


	Ft_App_WrCoCmd_Buffer(phost,VERTEX2F(8*(FT_DispWidth - aviw),8*(FT_DispHeight - avih)));//display at the center of the screen

	Ft_App_WrCoCmd_Buffer(phost,DISPLAY());
	Ft_Gpu_CoCmd_Swap(phost);
	Ft_App_Flush_Co_Buffer(phost);

	return 0;
}

#if defined(FT900_PLATFORM)
ft_int32_t LoadImageMFifo(ft_int32_t flags,Ft_Gpu_Hal_Context_t *phost,Ft_Fifo_t *pmfifo,ft_char8_t *filename)
{
	//load image via media fifo or via cmd fifo
	ft_int32_t fResult,filesz,currchunk;

	fResult = f_open(&CurFile, filename, FA_READ);

	if(FR_OK != fResult)
	{
		printf("Error in opening the file %s\n",filename);
		return -1;
	}

	filesz = f_size(&CurFile);
	while(filesz > 0)
	{
		ft_uint32_t bytesread;

		currchunk = Ft_Fifo_GetFreeSpace(phost,pmfifo);

		if(currchunk > FT_AVIPARSER_BUFFER_SIZE)
		{
			currchunk = FT_AVIPARSER_BUFFER_SIZE;
		}

		if(currchunk > filesz)
		{
			currchunk = filesz;
		}
		fResult = f_read(&CurFile,G_PBuff,currchunk,&bytesread);
		Ft_Gpu_Hal_StartTransfer(phost,FT_GPU_WRITE, pmfifo->fifo_buff + pmfifo->fifo_wp);
		spi_writen(SPIM,G_PBuff,bytesread);
		Ft_Gpu_Hal_EndTransfer(phost);
		pmfifo->fifo_wp += bytesread;
		filesz -= bytesread;
		Ft_Gpu_Hal_Wr32(phost,pmfifo->HW_Write_Reg,pmfifo->fifo_wp);
	}
	return 0;
}
#endif

ft_uint8_t keypressed = 0;
ft_uint8_t keyin_cts = 0;
#define KEYIN_COUNTS	(20)
ft_uint8_t Read_keys(ft_uint8_t ttag)
{
	static ft_uint8_t temp_key = 0,prev_keyin = 0;
	keypressed = ttag;//Ft_Gpu_Hal_Rd8(phost,REG_TOUCH_TAG);
	if(keypressed!=0 && temp_key!=keypressed)
	temp_key = keypressed;
	if(keypressed==0)
	{
		if(temp_key!=0)
		{
			prev_keyin = temp_key;
			temp_key = 0;
			return prev_keyin;
		}
		temp_key = 	keypressed;
	}
	return 0;
}


#define FT_MAINMENU_NUMFILES	(4)
#define FT_MAINMENU_WIDTH		(240)
#define FT_MAINMENU_HEIGHT		(144)
#define FT_MAINMENU_DISTANCE	(60)

//draw 4 thumbnails
ft_char8_t *pffname[] = {"Sintel","Tears of Steel","Elephants Dream","Big Buck Bunny"};
ft_char8_t thumbnail[] = "avip30p.jpg";
ft_int32_t MainMenu(FT_PGUI_t *pgui,Ft_Gpu_Hal_Context_t *phost,Ft_Fifo_t *pmfifo)
{

	ft_int32_t image_height = FT_MAINMENU_HEIGHT,image_width = FT_MAINMENU_WIDTH;
	ft_int32_t dt = FT_MAINMENU_DISTANCE,dx,dy;
	ft_int32_t per_frame,no_frames,key_in,current_frame;
	ft_int32_t sx,drag,Oy,Ox,dragth,i;
	//prolog state
	DefaultGUIScreen(phost);
	dx = (dt*2)+image_width;
	dy = (10*2)+image_height;
	per_frame = FT_DispWidth/dx;

	if(per_frame == 0)
		per_frame = 1;

	no_frames = (FT_MAINMENU_NUMFILES-1)/per_frame;
	//decode jpg picture
	Ox = FT_MAINMENU_WIDTH * FT_MAINMENU_HEIGHT * 2L * FT_MAINMENU_NUMFILES + 32L;//size of 4 is due to the result of video decode
	Oy = RAM_G_SIZE - Ox;
	/* Setup FT81x for video data path */
	Ft_Fifo_Init(pmfifo,Ox,Oy,REG_MEDIAFIFO_READ,REG_MEDIAFIFO_WRITE);
    Ft_Gpu_CoCmd_MediaFifo(phost,Ox,Oy);//address of the media fifo buffer - init of fifo done by coprocessor
    Ft_App_Flush_Co_Buffer(phost);

#if defined(FT900_PLATFORM)
    LoadImageMFifo(0,phost,pmfifo,thumbnail);
    Ft_Gpu_CoCmd_LoadImage(phost,0, OPT_MEDIAFIFO | OPT_NODL);
    Ft_App_Flush_Co_Buffer(phost);
    Ft_Gpu_Hal_WaitCmdfifo_empty(phost);
#endif
	while(1)
	{
		ft_int32_t directxy,pendown = 0,touchtag = 0,tagxy[2];
		/*Read touch screen x varaiation and tag in*/
		Ft_Gpu_Hal_RdMem(phost,REG_TOUCH_TAG_XY,tagxy,8);
		  sx =  tagxy[0] >> 16;
		  directxy = Ft_Gpu_Hal_Rd32(phost,REG_TOUCH_DIRECT_XY);
		  touchtag = tagxy[1];
		  if(0 == (directxy & 0x80000000))
		  {
			  pendown = 1;
		  }
		  key_in =  Read_keys(touchtag);
		  /*if tag in but still pendown take a scroller basevalue*/
		  if(keyin_cts>KEYIN_COUNTS)
		  {
			  key_in = 0;
		  }
		  if(key_in==0)
		  {
			  scroller_run(sx,directxy);
		  }
		  drag = scroller.base>>4;
		if(((key_in > 0) && (key_in <= FT_MAINMENU_NUMFILES)) && (NOTOUCH == sx) && (abs(scroller.vel) < 10*16))
		{
			printf("video selection %d \n",key_in);
			scroller.vel = 0;
			return (key_in - 1);
		}
		  /*Check if any tag in*/
		  if(sx!=NOTOUCH)
			  keyin_cts++;
		  /*Move into the particular frame based on dragdt now 30pixels*/
		  if(sx==NOTOUCH)
		  {
			  keyin_cts = 0;
		  }
		Ft_Gpu_CoCmd_Dlstart(phost);
		Ft_App_WrCoCmd_Buffer(phost,CLEAR_COLOR_RGB(0,0,0));
		Ft_App_WrCoCmd_Buffer(phost,CLEAR(1,1,1));
		Ft_App_WrCoCmd_Buffer(phost,BITMAP_HANDLE(1));
		Ft_App_WrCoCmd_Buffer(phost,BITMAP_SOURCE(0));
		Ft_App_WrCoCmd_Buffer(phost,BITMAP_LAYOUT(RGB565,FT_MAINMENU_WIDTH*2L,FT_MAINMENU_HEIGHT));
		Ft_App_WrCoCmd_Buffer(phost,BITMAP_SIZE(NEAREST,BORDER,BORDER,FT_MAINMENU_WIDTH,FT_MAINMENU_HEIGHT));
		Backgroundanimation_3();
		image_height = FT_MAINMENU_HEIGHT;
		image_width = FT_MAINMENU_WIDTH;
		Ft_App_WrCoCmd_Buffer(phost,TAG_MASK(1));
		Ft_App_WrCoCmd_Buffer(phost,COLOR_RGB(255,255,255));
		Ft_App_WrCoCmd_Buffer(phost,LINE_WIDTH(1*16)); // for rect
		Oy = (FT_DispHeight - image_height - 80)/2;					//dt+(dy*((i/col)%row));
		current_frame = drag/dx;                       // noof items moved in +/- directions
		dragth = drag%dx;
		Ft_App_WrCoCmd_Buffer(phost,BEGIN(RECTS));
		for(i=-1;i<(per_frame+1);i++)
		{
			Ox = dt+dx*i;
			Ox-=dragth;
			if(Ox > (FT_DispWidth + dt) || Ox < -dx) 0;
			else
			{
				Ft_App_WrCoCmd_Buffer(phost,VERTEX2F((Ox-1)*16,(Oy-1)*16));
				Ft_App_WrCoCmd_Buffer(phost,VERTEX2F((image_width+Ox+1)*16,(image_height+Oy+1)*16));					// i pixels wide than image width +1
			}
		}

		Ft_App_WrCoCmd_Buffer(phost,BEGIN(BITMAPS));
		// draw the bitmap

		for(i=-1;i<(per_frame+1);i++)
		{
			Ox = dt+dx*i;
			Ox-=dragth;
			if(Ox > (FT_DispWidth+dt) || Ox < -dx) 0;
			else
			{
				ft_int32_t cellno = ((FT_MAINMENU_NUMFILES+i+current_frame)%FT_MAINMENU_NUMFILES);
				Ft_App_WrCoCmd_Buffer(phost,BITMAP_HANDLE(1));
				Ft_App_WrCoCmd_Buffer(phost,CELL(cellno));
				Ft_App_WrCoCmd_Buffer(phost,TAG((1+i+current_frame)%(FT_MAINMENU_NUMFILES+1)));
				Ft_App_WrCoCmd_Buffer(phost,VERTEX2F(Ox*16,Oy*16));
				Ft_Gpu_CoCmd_Text(phost,Ox + FT_MAINMENU_WIDTH/2,Oy + FT_MAINMENU_HEIGHT + 20,27,OPT_CENTERX|OPT_CENTERY,pffname[cellno]);

			}
		}

		Ft_App_WrCoCmd_Buffer(phost,DISPLAY());
		Ft_Gpu_CoCmd_Swap(phost);
		Ft_App_Flush_Co_Buffer(phost);
		Ft_Gpu_Hal_WaitCmdfifo_empty(phost);

	}

	return 0;
}



/* Global definitions - to be migrated to dynamic allocation based on input file */
FT_PGUI_t G_S_PGui;
FT_AVIPlayer_t G_S_AviPlayer;
FT_AVIParser_t G_S_AviParser;
FT_VidDataPath_t G_SVidDataPath;
FT_AudDataPath_t G_SAudDataPath;
Ft_Fifo_t stFifo;
FT_RingQueue_t srqaudin,srqaudout,srqvidin,srqvidout;
FT_RingBuffer_t srbaudin,srbaudout;

#define FT_AVIPLAYER_MAX_AUDIN_QE	(1)
#define FT_AVIPLAYER_MAX_AUDOUT_QE	(1)
#define FT_AVIPLAYER_MAX_VIDIN_QE	(20)
#define FT_AVIPLAYER_MAX_VIDOUT_QE	(20)


#define FT_AVIPARSER_TESTSTUB_VIDBUFFMAXSZ	(200*1024)

/* files to be tested */
#define FT_PLAYER_TESTNUM	(4)

#if defined(MSVC_PLATFORM) || defined(MSVC_FT800EMU)
	#define FT_GUI_DISAPPEAR_COUNT	(20)
#elif defined(FT900_PLATFORM)
	#define FT_GUI_DISAPPEAR_COUNT	(120000)
#endif


//ft_char8_t *pfilenames[10] = {"csf_30.avi","cest1.avi","cest1pcm.avi","csf.avi","cest.avi","csfsvga.avi","cinsvga.avi"};
#if defined(MSVC_PLATFORM) || defined(MSVC_FT800EMU)
ft_char8_t *pfilenames[10] = {"..\\..\\..\\TEST\\Sintel.avi","..\\..\\..\\TEST\\TOS.avi","..\\..\\..\\TEST\\ED.avi","..\\..\\..\\TEST\\BBB_16.avi"};
#elif defined(FT900_PLATFORM)
ft_char8_t *pfilenames[10] = {"Sintel.avi","TOS.avi","ED.avi","BBB_16.avi"};
#endif
/* Font array */
/* VC1 Hardware Fonts index 28*/
FT_Gpu_Fonts_t Inbuilt28Font = {8,8,8,8,8,8,8,8,8,8,8,8,8,8,8,8,8,8,8,8,8,8,8,8,8,8,8,8,8,8,8,8,8,8,8,8,8,8,8,8,8,8,8,8,8,8,8,8,8,8,8,8,8,8,8,8,8,8,8,8,8,8,8,8,8,8,8,8,8,8,8,8,8,8,8,8,8,8,8,8,8,8,8,8,8,8,8,8,8,8,8,8,8,8,8,8,8,8,8,8,8,8,8,8,8,8,8,8,8,8,8,8,8,8,8,8,8,8,8,8,8,8,8,8,8,8,8,8,
		1,
		1,
		8,
		8,
		1047548};

/* HAL context */
//Ft_Gpu_Hal_Context_t host,*phost;
//gui context

/* Ring queue buffer */
FT_MetaV_t G_SA_VMetaDInRQ[FT_AVIPLAYER_MAX_VIDIN_QE];
FT_MetaV_t G_SA_VMetaDOutRQ[FT_AVIPLAYER_MAX_VIDOUT_QE];


FT_MetaA_t G_SA_AMetaDInRQ[FT_AVIPLAYER_MAX_AUDIN_QE];
FT_MetaA_t G_SA_AMetaDOutRQ[FT_AVIPLAYER_MAX_AUDOUT_QE];



ft_int32_t tempformtext28(ft_char8_t *pstring,ft_int32_t xoffset,ft_int32_t yoffset)
{
	ft_int32_t tidx,tlen;
	ft_char8_t charcurr;

	tlen = strlen(pstring);

	for(tidx = 0;tidx<tlen;tidx++)
	{
		charcurr = *pstring++;
		Ft_App_WrDlCmd_Buffer(phost,CELL(charcurr));
		Ft_App_WrDlCmd_Buffer(phost,VERTEX2F(16*(xoffset),16*(yoffset)));
		xoffset += Inbuilt28Font.FontWidth[charcurr];
	}

	return 0;
}

volatile ft_uint8_t parserfillmutex = 0,videodecodemutex = 0,guimutex = 0;

ft_void_t ft_timer_interrupt1()
{

#if defined(FT900_PLATFORM)
	/* clear the pending interrupt */
	timer_is_interrupted(timer_select_a);
#endif

{
#if defined(FT900_PLATFORM)
	/* Increment the global isr counter */
	FTPayer_TMicroS += 1;
	/* Disable the timer interrupt in order not to receive any further interrupts */
	timer_disable_interrupt(timer_select_a);
#endif
	//videodecodemutex = 0;

	if((1 == parserfillmutex) || (1 == guimutex))
	{
		videodecodemutex++;
	}
	else
	{
		if(Ft_Gpu_Hal_Rd16(phost,REG_CMD_READ) == Ft_Gpu_Hal_Rd16(phost,REG_CMD_WRITE))
		{
			if(srqvidin.ValidEle >= 1)
			{
				ft_uint8_t valideles = srqvidin.ValidEle;
				FT_MetaV_t tempvmeta;

				/* Read the element and trigger a decode */
				//printf("ft81x status bfr %x %x %x %x \n",Ft_Gpu_Hal_Rd32(phost,REG_CMD_READ),Ft_Gpu_Hal_Rd32(phost,REG_CMD_WRITE),Ft_Gpu_Hal_Rd32(phost,REG_MEDIAFIFO_READ),Ft_Gpu_Hal_Rd32(phost,REG_MEDIAFIFO_WRITE));
				FT_RQueue_PeekE(&srqvidin,&tempvmeta,srqvidin.ReadIdx);
				if(0 == tempvmeta.PartialSz)
				{
					FT_RQueue_ReadE(&srqvidin,&tempvmeta);
					if(tempvmeta.Length > 0)
					{
						Ft_Gpu_CoCmd_LoadImage(phost,0, OPT_MEDIAFIFO | OPT_NODL);
						Ft_App_Flush_Co_Buffer(phost);
						G_SVidDataPath.NumDecoded++;
					}
					G_S_AviPlayer.TotalBytes += tempvmeta.Length;//total number of bytes consumed till now by decoder
				}
			}

		}
	}
#if defined(FT900_PLATFORM)
	timer_enable_interrupt(timer_select_a);
#endif
}
}

ft_void_t calculateFrameRotatialScaling(FT_VIDEO_FRAME *videoFrame){
	ft_float_t horizontalScaleVal=1, verticalScaleVal=1;
	ft_uint16_t tempVal;

	//if(videoFrame->rotation == FT_VIDEO_PORTRAIT || videoFrame->rotation == FT_VIDEO_INVERTED_PORTRAIT || videoFrame->rotation == FT_VIDEO_MIRRORED_PORTRAIT || videoFrame->rotation == FT_VIDEO_MIRROED_INVERTED_PORTRAIT){ //frame is horizontally positioned
	//if(videoFrame->rotationVal == 90){ //frame is horizontally positioned
	if(0){	
		if(videoFrame->frameWidth > FT_DispHeight){
			verticalScaleVal =  (ft_float_t)FT_DispHeight / (ft_float_t)videoFrame->frameWidth;
		}

		if(videoFrame->frameHeight > FT_DispWidth){
			horizontalScaleVal =  (ft_float_t)FT_DispWidth / (ft_float_t)videoFrame->frameHeight;
		}

		if(horizontalScaleVal == verticalScaleVal){
			videoFrame->scaleFactor = 65535;
		}
		else if(horizontalScaleVal < verticalScaleVal){
			videoFrame->scaleFactor = (ft_float_t)horizontalScaleVal * 65535;
			videoFrame->finalFrameHeight = (ft_int16_t)((ft_float_t)videoFrame->frameWidth * horizontalScaleVal);
			videoFrame->finalFrameWidth= (ft_int16_t)((ft_float_t)videoFrame->frameHeight * horizontalScaleVal);
		}
		else if(verticalScaleVal < horizontalScaleVal){
			videoFrame->scaleFactor = (ft_float_t)verticalScaleVal * 65535;
			videoFrame->finalFrameHeight = (ft_int16_t)((ft_float_t)videoFrame->frameWidth * verticalScaleVal);
			videoFrame->finalFrameWidth = (ft_int16_t)((ft_float_t)videoFrame->frameHeight * verticalScaleVal);
		}
	}else{
		if(videoFrame->frameWidth > FT_DispWidth){
			horizontalScaleVal =  (ft_float_t)FT_DispWidth / (ft_float_t)videoFrame->frameWidth;
		}

		if(videoFrame->frameHeight > FT_DispHeight){
			verticalScaleVal =  (ft_float_t)FT_DispHeight / (ft_float_t)videoFrame->frameHeight;
		}


		if(horizontalScaleVal == verticalScaleVal){
			videoFrame->scaleFactor = 65535;
		}
		else if(horizontalScaleVal < verticalScaleVal){
			videoFrame->scaleFactor = (ft_float_t)horizontalScaleVal * 65535;
			videoFrame->finalFrameWidth = (ft_int16_t)((ft_float_t)videoFrame->frameWidth * horizontalScaleVal);
			videoFrame->finalFrameHeight= (ft_int16_t)((ft_float_t)videoFrame->frameHeight * horizontalScaleVal);
		}
		else if(verticalScaleVal < horizontalScaleVal){
			videoFrame->scaleFactor = (ft_float_t)verticalScaleVal * 65535;
			videoFrame->finalFrameWidth = (ft_int16_t)((ft_float_t)videoFrame->frameWidth * verticalScaleVal);
			videoFrame->finalFrameHeight = (ft_int16_t)((ft_float_t)videoFrame->frameHeight * verticalScaleVal);
		}
		videoFrame->zoomOut=0;
	}
	/*
	Ft_Gpu_CoCmd_LoadIdentity(phost);
	Ft_Gpu_CoCmd_Scale(phost, commonScaleVal, commonScaleVal);
	Ft_Gpu_CoCmd_SetMatrix(phost);
	*/
}





ft_void_t setControlButtonsPosition(FT_VIDEO_FRAME *videoFrame){
	FT_GUI_BUTTONS_POS *controlButtons = videoFrame->controlButtons;
	ft_uint16_t singleBlockWidth = FT_DispWidth / 20.0;
	printf("singleBlockWidth: %u\n",singleBlockWidth);
	controlButtons->playYPos = controlButtons->statusYPos = controlButtons->exitYPos = controlButtons->rotateYPos = FT_DispHeight * 0.80;
	printf("playYPos: %u\n",controlButtons->playYPos);
	controlButtons->playXPos = singleBlockWidth;
	printf("playXPos: %u\n",controlButtons->playXPos);
	controlButtons->playWidth = singleBlockWidth * 2;
	printf("playWidth: %u\n",controlButtons->playWidth);
	controlButtons->statusXPos = singleBlockWidth * 4;
	printf("statusXPos: %u\n",controlButtons->statusXPos);
	controlButtons->statusWidth = singleBlockWidth * 10;
	printf("statusWidth: %u\n",controlButtons->statusWidth);
	controlButtons->exitXPos = singleBlockWidth * 15;
	printf("exitXPos: %u\n",controlButtons->exitXPos);
	controlButtons->exitWidth = singleBlockWidth * 2;
	printf("exitWidth: %u\n",controlButtons->exitWidth);
	controlButtons->rotateXPos = singleBlockWidth * 18;
	printf("rotateXPos: %u\n",controlButtons->rotateXPos);
	controlButtons->rotateWidth = singleBlockWidth * 2;
	printf("rotateWidth: %u\n",controlButtons->rotateWidth);
	controlButtons->buttonHeight = singleBlockWidth*2;
	printf("buttonHeight: %u\n",controlButtons->buttonHeight);
}

ft_void_t rotateScreen(FT_VIDEO_FRAME *videoFrame, FT_VIDEO_ORIENTATION orientation){
	ft_uint16_t tempVal;

	if(videoFrame->rotation != orientation){
		if(orientation == FT_VIDEO_PORTRAIT || orientation == FT_VIDEO_INVERTED_PORTRAIT || orientation == FT_VIDEO_MIRRORED_PORTRAIT || videoFrame->rotation == FT_VIDEO_MIRROED_INVERTED_PORTRAIT){
			tempVal = FT_DispWidth;
			FT_DispWidth = FT_DispHeight;
			FT_DispHeight = tempVal;
		}
		else{
			tempVal = FT_DispWidth;
			FT_DispWidth = FT_DispHeight;
			FT_DispHeight = tempVal;
		}
		videoFrame->rotation = orientation;
		calculateFrameRotatialScaling(videoFrame);
		setControlButtonsPosition(videoFrame);
	}
	Ft_Gpu_CoCmd_SetRotate(phost,orientation);
}

ft_void_t applyFrameRotationalScaling(FT_VIDEO_FRAME *videoFrame, FT_VIDEO_ORIENTATION orientation){
	rotateScreen(videoFrame, orientation);
	if(videoFrame->scaleFactor != 65535){
		Ft_Gpu_CoCmd_LoadIdentity(phost);
			//Ft_Gpu_CoCmd_Translate(phost, 65535 * videoFrame->frameWidth / 2, 65535 * videoFrame->frameHeight / 2); //make the rotation coordinates at the center
			//Ft_Gpu_CoCmd_Rotate(phost, videoFrame->rotationVal * 65535 / 360); 
			Ft_Gpu_CoCmd_Scale(phost, videoFrame->scaleFactor, videoFrame->scaleFactor);			
			//Ft_Gpu_CoCmd_Translate(phost, -65535 * videoFrame->frameWidth / 2, -65535 * videoFrame->frameHeight / 2);
		Ft_Gpu_CoCmd_SetMatrix(phost);
	}
}

ft_void_t restoreFrameRotationalScaling(FT_VIDEO_FRAME *videoFrame){
		videoFrame->scaleFactor = 65535;
		applyFrameRotationalScaling(videoFrame, FT_VIDEO_LANDSCAPE);
		Ft_Gpu_CoCmd_LoadIdentity(phost);
		Ft_Gpu_CoCmd_SetMatrix(phost);
		Ft_Gpu_CoCmd_SetRotate(phost,0);
}

ft_void_t drawControlButtons(FT_GUI_BUTTONS_POS *controlButtons){

							//draw three rectangles, 1 triangle, 2 lines,
						Ft_App_WrCoCmd_Buffer(phost,TAG_MASK(1));
						Ft_App_WrCoCmd_Buffer(phost,COLOR_RGB(32,32,32));  //revert
						//Ft_App_WrCoCmd_Buffer(phost,COLOR_RGB(0,255,0));
						Ft_App_WrCoCmd_Buffer(phost,COLOR_A(128));
						Ft_App_WrCoCmd_Buffer(phost,LINE_WIDTH(1*16));
						Ft_App_WrCoCmd_Buffer(phost,BEGIN(RECTS));

						Ft_App_WrCoCmd_Buffer(phost,COLOR_RGB(0,0,64));
						Ft_App_WrCoCmd_Buffer(phost,VERTEX2F(16*(controlButtons->statusXPos ),16*(controlButtons->statusYPos)));
						Ft_App_WrCoCmd_Buffer(phost,VERTEX2F(16*(controlButtons->statusXPos + controlButtons->statusWidth),16*(controlButtons->statusYPos + controlButtons->buttonHeight)));
						
						//Ft_App_WrCoCmd_Buffer(phost,VERTEX2F(16*(controlButtons->statusXPos),16*(controlButtons->statusYPos)));
						//Ft_App_WrCoCmd_Buffer(phost,VERTEX2F(16*(controlButtons->statusXPos + controlButtons->statusWidth),16*(controlButtons->statusYPos + controlButtons->buttonHeight)));

						Ft_App_WrCoCmd_Buffer(phost,COLOR_RGB(32,32,32)); //revert
						//play/pause rectangle
						Ft_App_WrCoCmd_Buffer(phost,TAG(1));
						Ft_App_WrCoCmd_Buffer(phost,VERTEX2F(16*(controlButtons->playXPos),16*(controlButtons->playYPos)));
						Ft_App_WrCoCmd_Buffer(phost,VERTEX2F(16*(controlButtons->playXPos + controlButtons->playWidth),16*(controlButtons->playYPos + controlButtons->playWidth)));

						if(G_S_PGui.TouchSel & 0x02)
						{
							Ft_App_WrCoCmd_Buffer(phost,COLOR_A(255));
							Ft_App_WrCoCmd_Buffer(phost,VERTEX2F(16*(controlButtons->playXPos),16*(controlButtons->playYPos)));
							Ft_App_WrCoCmd_Buffer(phost,VERTEX2F(16*(controlButtons->playXPos + controlButtons->playWidth),16*(controlButtons->playYPos + controlButtons->playWidth)));
							Ft_App_WrCoCmd_Buffer(phost,COLOR_A(128));
						}

						//exit rectangle					
						Ft_App_WrCoCmd_Buffer(phost,TAG(2));
						Ft_App_WrCoCmd_Buffer(phost,VERTEX2F(16*(controlButtons->exitXPos),16*(controlButtons->exitYPos)));
						Ft_App_WrCoCmd_Buffer(phost,VERTEX2F(16*(controlButtons->exitXPos + controlButtons->exitWidth),16*(controlButtons->exitYPos + controlButtons->exitWidth)));

						Ft_App_WrCoCmd_Buffer(phost,COLOR_A(255));
						if(G_S_PGui.TouchSel & 0x04)
						{
							Ft_App_WrCoCmd_Buffer(phost,COLOR_A(255));
							Ft_App_WrCoCmd_Buffer(phost,VERTEX2F(16*(controlButtons->exitXPos),16*(controlButtons->exitYPos)));
							Ft_App_WrCoCmd_Buffer(phost,VERTEX2F(16*(controlButtons->exitXPos + controlButtons->exitWidth),16*(controlButtons->exitYPos + controlButtons->exitWidth)));
						}

						//rotate button
						//exit rectangle					
						Ft_App_WrCoCmd_Buffer(phost,TAG(3));
						Ft_App_WrCoCmd_Buffer(phost,VERTEX2F(16*(controlButtons->rotateXPos),16*(controlButtons->rotateYPos)));
						Ft_App_WrCoCmd_Buffer(phost,VERTEX2F(16*(controlButtons->rotateXPos + controlButtons->rotateWidth),16*(controlButtons->rotateYPos + controlButtons->rotateWidth)));


						//triangle for play and two lines for pause
						Ft_App_WrCoCmd_Buffer(phost,TAG(1));
						Ft_App_WrCoCmd_Buffer(phost,COLOR_RGB(255,255,255));
						if(0x00 == (G_S_PGui.TouchSel & 0x01))
						{
							//pause button
							Ft_App_WrCoCmd_Buffer(phost,LINE_WIDTH(4*16));
							Ft_App_WrCoCmd_Buffer(phost,BEGIN(LINES));

							Ft_App_WrCoCmd_Buffer(phost,VERTEX2F(16*(controlButtons->playXPos + 8),16*(controlButtons->playYPos + 8)));
							Ft_App_WrCoCmd_Buffer(phost,VERTEX2F(16*(controlButtons->playXPos + 8),16*(controlButtons->playYPos + controlButtons->playWidth - 8)));

							Ft_App_WrCoCmd_Buffer(phost,VERTEX2F(16*(controlButtons->playXPos + controlButtons->playWidth - 8),16*(controlButtons->playYPos + 8)));
							Ft_App_WrCoCmd_Buffer(phost,VERTEX2F(16*(controlButtons->playXPos + controlButtons->playWidth - 8),16*(controlButtons->playYPos + controlButtons->playWidth - 8)));
						}
						else
						{
#if 0
							Ft_App_WrCoCmd_Buffer(phost,VERTEX2F(16*(controlButtons->playXPos + 8),16*(controlButtons->playYPos + 8)));
							Ft_App_WrCoCmd_Buffer(phost,VERTEX2F(16*(controlButtons->playXPos + controlButtons->playWidth - 8),16*(controlButtons->playYPos + controlButtons->playWidth - 8)));
							

							Ft_App_WrCoCmd_Buffer(phost,VERTEX2F(16*(controlButtons->playXPos + controlButtons->playWidth - 8),16*(controlButtons->playYPos + 8)));
							Ft_App_WrCoCmd_Buffer(phost,VERTEX2F(16*(controlButtons->playXPos + 8),16*(controlButtons->playYPos + controlButtons->playWidth - 8)));
#else
							//play button
							Ft_App_WrCoCmd_Buffer(phost,LINE_WIDTH(1*16));
							Ft_App_WrCoCmd_Buffer(phost,BEGIN(EDGE_STRIP_L));
							Ft_App_WrCoCmd_Buffer(phost,SCISSOR_XY(controlButtons->playXPos, controlButtons->playYPos));
							Ft_App_WrCoCmd_Buffer(phost,SCISSOR_SIZE(controlButtons->playWidth,controlButtons->playWidth));

							Ft_App_WrCoCmd_Buffer(phost,COLOR_MASK(0,0,0,1));
							Ft_App_WrCoCmd_Buffer(phost,CLEAR(1,0,0));
							Ft_App_WrCoCmd_Buffer(phost,STENCIL_OP(INCR,INCR));

							Ft_App_WrCoCmd_Buffer(phost,VERTEX2F(16*(controlButtons->playXPos + 6),16*(controlButtons->playYPos + 6)));
							Ft_App_WrCoCmd_Buffer(phost,VERTEX2F(16*(controlButtons->playXPos + controlButtons->playWidth - 6),16*(controlButtons->playYPos + controlButtons->playWidth / 2)));
							Ft_App_WrCoCmd_Buffer(phost,VERTEX2F(16*(controlButtons->playXPos + 6),16*(controlButtons->playYPos + controlButtons->playWidth - 6)));
							Ft_App_WrCoCmd_Buffer(phost,VERTEX2F(16*(controlButtons->playXPos + 6),16*(controlButtons->playYPos + 6)));
							Ft_App_WrCoCmd_Buffer(phost,END());
							Ft_App_WrCoCmd_Buffer(phost,BEGIN(EDGE_STRIP_L));
							Ft_App_WrCoCmd_Buffer(phost,STENCIL_FUNC(EQUAL,1,255));
							Ft_App_WrCoCmd_Buffer(phost,COLOR_MASK(1,1,1,1));
							Ft_App_WrCoCmd_Buffer(phost,VERTEX2F(16*(controlButtons->playXPos + controlButtons->playWidth),16*(controlButtons->playYPos)));
							Ft_App_WrCoCmd_Buffer(phost,VERTEX2F(16*(controlButtons->playXPos + controlButtons->playWidth),16*(controlButtons->playYPos + controlButtons->playWidth)));

							Ft_App_WrCoCmd_Buffer(phost,STENCIL_FUNC(ALWAYS,1,255));
							Ft_App_WrCoCmd_Buffer(phost,BEGIN(LINE_STRIP));
							Ft_App_WrCoCmd_Buffer(phost,VERTEX2F(16*(controlButtons->playXPos + 6),16*(controlButtons->playYPos + 6)));
							Ft_App_WrCoCmd_Buffer(phost,VERTEX2F(16*(controlButtons->playXPos + controlButtons->playWidth - 6),16*(controlButtons->playYPos + controlButtons->playWidth /2)));
							Ft_App_WrCoCmd_Buffer(phost,VERTEX2F(16*(controlButtons->playXPos + 6),16*(controlButtons->playYPos + controlButtons->playWidth - 6)));
							Ft_App_WrCoCmd_Buffer(phost,VERTEX2F(16*(controlButtons->playXPos + 6),16*(controlButtons->playYPos + 6)));
							Ft_App_WrCoCmd_Buffer(phost,SCISSOR_XY(0,0));
							Ft_App_WrCoCmd_Buffer(phost,SCISSOR_SIZE(FT_DispWidth,FT_DispHeight));
#endif
						}

						//x for exit button
						Ft_App_WrCoCmd_Buffer(phost,TAG(2));
						Ft_App_WrCoCmd_Buffer(phost,LINE_WIDTH(2*16));
						Ft_App_WrCoCmd_Buffer(phost,BEGIN(LINES));

						Ft_App_WrCoCmd_Buffer(phost,VERTEX2F(16*(controlButtons->exitXPos + 6),16*(controlButtons->exitYPos + 6)));
						Ft_App_WrCoCmd_Buffer(phost,VERTEX2F(16*(controlButtons->exitXPos + controlButtons->exitWidth - 6),16*(controlButtons->exitYPos + controlButtons->exitWidth - 6)));
							
						Ft_App_WrCoCmd_Buffer(phost,VERTEX2F(16*(controlButtons->exitXPos + controlButtons->exitWidth - 6),16*(controlButtons->exitYPos + 6)));
						Ft_App_WrCoCmd_Buffer(phost,VERTEX2F(16*(controlButtons->exitXPos + 6),16*(controlButtons->exitYPos + controlButtons->exitWidth - 6)));

						//status bar
						Ft_App_WrCoCmd_Buffer(phost,TAG_MASK(0));
						Ft_App_WrCoCmd_Buffer(phost,LINE_WIDTH(8*16));
						//Ft_App_WrCoCmd_Buffer(phost,COLOR_RGB(0,0,64));
						//Ft_App_WrCoCmd_Buffer(phost,VERTEX2F(16*(controlButtons->statusXPos ),16*(controlButtons->statusYPos)));
						//Ft_App_WrCoCmd_Buffer(phost,VERTEX2F(16*(controlButtons->statusXPos + controlButtons->statusWidth),16*(controlButtons->statusYPos + controlButtons->buttonHeight)));
						Ft_App_WrCoCmd_Buffer(phost,COLOR_RGB(0,0,192));
						Ft_App_WrCoCmd_Buffer(phost,VERTEX2F(16*(controlButtons->statusXPos + 10),16*(controlButtons->statusYPos + (controlButtons->buttonHeight / 2))));
						Ft_App_WrCoCmd_Buffer(phost,VERTEX2F(16*(controlButtons->statusXPos + 10 + (G_S_AviParser.SVid.CurrFrame*((ft_uint32_t)controlButtons->statusWidth-20))/G_S_AviParser.SVid.TotFrames),16*(controlButtons->statusYPos + controlButtons->buttonHeight / 2)));

						Ft_App_WrCoCmd_Buffer(phost,TAG_MASK(1));
						}

				





/* Testing avi parser */
#if defined MSVC_PLATFORM | defined FT900_PLATFORM
/* Main entry point */
ft_int32_t main(ft_int32_t argc,ft_char8_t *argv[])
#endif
#if defined(ARDUINO_PLATFORM)||defined(MSVC_FT800EMU)
ft_void_t setup()
#endif
{
	ft_uint32_t i,j,k = 0;
	Ft_Gpu_HalInit_t halinit;
	ft_uint32_t mediafifo,mediafifolen;
	ft_int32_t aviw,avih;
	ft_uint8_t frameRotationalScalingApplied = 0;
	ft_uint32_t prevSelect=0;
	FT_VIDEO_FRAME videoFrame;
	FT_GUI_BUTTONS_POS controlButtons;
	FT_VIDEO_ORIENTATION curOrientation=FT_VIDEO_LANDSCAPE;
	videoFrame.controlButtons = &controlButtons;
	videoFrame.rotation = FT_VIDEO_LANDSCAPE;
	/* Initialize all the global contexts */
	G_S_PGui.CurrTTag = 0;
	G_S_PGui.PrevTTag = 0;
	G_S_PGui.TouchSel = 0;//max fps
	G_S_PGui.VRateK = 0;
	G_S_PGui.Flags = FT_GUI_NONE;
	G_S_PGui.Disappear = FT_GUI_DISAPPEAR_COUNT;


	/* Player context */
	G_S_AviPlayer.AudioCodec = 0;
	G_S_AviPlayer.CurrTime = 0;
	G_S_AviPlayer.StateCurr = FT_AVIPLAYER_INIT;//init state
	G_S_AviPlayer.StateNext = 0;//none
	G_S_AviPlayer.VideoCodec = 0;
	G_S_AviPlayer.TotalBytes = 0;//total number of bytes consumed till now

	/* Video data path context */
	G_SVidDataPath.pfifo = &stFifo;
	G_SVidDataPath.phalctxt = &host;
	G_SVidDataPath.pvirq = &srqvidin;
	G_SVidDataPath.pvorb = NULL;
	G_SVidDataPath.pvorq = &srqvidout;
	G_SVidDataPath.NumDecoded = 0;


	/* Audio data path context */
	G_SAudDataPath.pairb = NULL;
	G_SAudDataPath.pairq = &srqaudin;
	G_SAudDataPath.paorb = NULL;
	G_SAudDataPath.paorq = &srqaudout;

#ifdef FT900_PLATFORM
	FT900_Config();

	TIMER->TIMER_CONTROL_0 = 0x03;
    TIMER->TIMER_CONTROL_0 = 0x02;
#endif

	halinit.TotalChannelNum = 1;
	Ft_Gpu_Hal_Init(&halinit);
	host.hal_config.channel_no = 0;
	host.hal_config.pdn_pin_no = FT800_PD_N;
	host.hal_config.spi_cs_pin_no = FT800_SEL_PIN;
	Ft_Gpu_Hal_Open(&host);
	phost = &host;
	Ft_BootupConfig();
	#ifdef FT900_PLATFORM
		printf("\n reg_touch_rz =0x%x ", Ft_Gpu_Hal_Rd16(phost, REG_TOUCH_RZ));
		printf("\n reg_touch_rzthresh =0x%x ", Ft_Gpu_Hal_Rd32(phost, REG_TOUCH_RZTHRESH));
	    printf("\n reg_touch_tag_xy=0x%x",Ft_Gpu_Hal_Rd32(phost, REG_TOUCH_TAG_XY));
		printf("\n reg_touch_tag=0x%x",Ft_Gpu_Hal_Rd32(phost, REG_TOUCH_TAG));
	#endif
	printf("Scroller init %d \n",(((FT_MAINMENU_DISTANCE*2)+FT_MAINMENU_WIDTH)*FT_MAINMENU_NUMFILES - FT_DispWidth)*16);
	scroller_init((((FT_MAINMENU_DISTANCE*2)+FT_MAINMENU_WIDTH)*FT_MAINMENU_NUMFILES - FT_DispWidth)*16);
    /*It is optional to clear the screen here*/
    Ft_Gpu_Hal_WrMem(phost, RAM_DL,(ft_uint8_t *)FT_DLCODE_BOOTUP,sizeof(FT_DLCODE_BOOTUP));
    Ft_Gpu_Hal_Wr8(phost, REG_DLSWAP,DLSWAP_FRAME);
    Ft_Gpu_Hal_Sleep(100);//Show the booting up screen.
	/* GUI Init */
	home_setup();
	Info();
	while(1)
	{
	k = MainMenu(&G_S_PGui, &host,&stFifo);
	/* profiling */

	if((k < 0) || (k >= FT_PLAYER_TESTNUM))
	{
		k=0;
	}
	//for(k=0;k<FT_PLAYER_TESTNUM;k++)
	parserfillmutex = 0;videodecodemutex = 0;guimutex = 0;
	while(1)
	{
		ft_uint32_t resultpar,tempvar = 0;;

		/* Initialize few of the global parameters */

		G_S_AviPlayer.TotalBytes = 0;
		G_S_PGui.Flags &= (~FT_GUI_REFRESH);
		G_SVidDataPath.NumDecoded = 0;
		FT_RQueue_Init(&srqvidin,FT_AVIPLAYER_MAX_VIDIN_QE,(ft_uint8_t *)G_SA_VMetaDInRQ,sizeof(FT_MetaV_t));
		FT_RQueue_Init(&srqvidout,FT_AVIPLAYER_MAX_VIDOUT_QE,(ft_uint8_t *)G_SA_VMetaDOutRQ,sizeof(FT_MetaV_t));

		FT_RQueue_Init(&srqaudin,FT_AVIPLAYER_MAX_AUDIN_QE,(ft_uint8_t *)G_SA_AMetaDInRQ,sizeof(FT_MetaA_t));
		FT_RQueue_Init(&srqaudout,FT_AVIPLAYER_MAX_AUDOUT_QE,(ft_uint8_t *)G_SA_AMetaDOutRQ,sizeof(FT_MetaA_t));

		FT_RingBuffer_Init(&srbaudin, NULL,10*1024);//as of now hack
		FT_RingBuffer_Init(&srbaudout, NULL,10*1024);//as of now hack

		/* Parser testing */
		FT_AVIParser_Init(&G_S_AviParser);
		FT_AVIParser_SetBuff(&G_S_AviParser,G_PBuff,FT_AVIPARSER_BUFFER_SIZE);
		resultpar = FT_AVIParser_SetOpenFile(&G_S_AviParser,pfilenames[k]);
		if(resultpar != FT_AVIPARSER_OK)
		{
			continue;
		}
		FT_AVIParser_ParseAvi(&G_S_AviParser);

		/* GUI init */
		/* construct the DL and display */
		DefaultGUIScreen(phost);
		
		/* Video decoder settings */
		/* start video playback, load the data into media fifo */
		aviw = G_S_AviParser.SVid.Width;
		avih = G_S_AviParser.SVid.Height;
		mediafifo = aviw * avih * 2L + 32L;//size of 4 is due to the result of video decode
		mediafifolen = RAM_G_SIZE - mediafifo;
		
		/* Setup FT81x for video data path */
		Ft_Fifo_Init(&stFifo,mediafifo,mediafifolen,REG_MEDIAFIFO_READ,REG_MEDIAFIFO_WRITE);
		
        Ft_Gpu_CoCmd_MediaFifo(phost,mediafifo,mediafifolen);//address of the media fifo buffer - init of fifo done by coprocessor

        Ft_App_Flush_Co_Buffer(phost);
        Ft_Gpu_Hal_WaitCmdfifo_empty(phost);

        printf("media fifo init done %d %d \n",mediafifo,mediafifolen);
		
    	/* update the read and write pointers of mediafifo */
    	Ft_Gpu_Hal_Wr32(phost,stFifo.HW_Write_Reg,stFifo.fifo_wp);
    	printf("Media fifo rd wr %x %x %x %x \n",Ft_Gpu_Hal_Rd32(phost,stFifo.HW_Read_Reg),Ft_Gpu_Hal_Rd32(phost,stFifo.HW_Write_Reg),Ft_Gpu_Hal_Rd32(phost,REG_CMD_READ),Ft_Gpu_Hal_Rd32(phost,REG_CMD_WRITE));

#if defined(FT900_PLATFORM)
    	printf("Start playback of new video %s %x %x %x %x %x %x \n", pfilenames[k], fResult,Ft_Gpu_Hal_Rd32(phost,REG_CMD_READ),Ft_Gpu_Hal_Rd32(phost,REG_CMD_WRITE),Ft_Gpu_Hal_Rd32(phost,REG_MEDIAFIFO_READ),Ft_Gpu_Hal_Rd32(phost,REG_MEDIAFIFO_WRITE),stFifo.fifo_wp);
#endif

		/* playback video only */
		FT_AVIParser_SetRiff(&G_S_AviParser,0);

		/* Initial filing of input data */
		resultpar = FT_AVIPARSER_OK;
		for(i=0;i<10;i++)
		{
			resultpar = FT_AVIParser_FillChunk(&G_S_AviParser,&G_SVidDataPath,&G_SAudDataPath);
			if(resultpar != FT_AVIPARSER_OK)
			{
				printf("Initial jitter buffer fill  %x %x %x %x %x \n", Ft_Gpu_Hal_Rd32(phost,REG_CMD_READ),Ft_Gpu_Hal_Rd32(phost,REG_CMD_WRITE),Ft_Gpu_Hal_Rd32(phost,REG_MEDIAFIFO_READ),Ft_Gpu_Hal_Rd32(phost,REG_MEDIAFIFO_WRITE),stFifo.fifo_wp);
				break;
			}
		}

#if defined(FT900_PLATFORM)
		/* Steady state of avi player */
		/* start timer for new stream */
		timercount = 0;
		timervalue = 0;
		/* Timer for fps */
		ft_timer_interrupt1_exit();//exit the earlier timer
		timercount = 0,timervalue = 0;
		tempvar = G_S_AviParser.SVid.DataRate/G_S_AviParser.SVid.TimeScale;
		tempvar = 1000000/tempvar;
		printf("Video timer start fps tval %d %d \n",tempvar,1000);
		ft_timer_interrupt1_init((tempvar),100);//start the timer fresh
		printf("after init\n");
#endif
		G_S_PGui.Flags |= FT_GUI_REFRESH;
		G_S_PGui.TouchSel = 0;
		G_S_PGui.PrevTTag = 0;
		G_S_PGui.Disappear = FT_GUI_DISAPPEAR_COUNT;
		G_S_AviPlayer.StateCurr = FT_AVIPLAYER_PLAY;


		videoFrame.frameWidth = G_S_AviParser.SVid.Width;
		videoFrame.frameHeight = G_S_AviParser.SVid.Height;
		rotateScreen(&videoFrame, FT_VIDEO_LANDSCAPE);
		curOrientation = FT_VIDEO_LANDSCAPE;
		calculateFrameRotatialScaling(&videoFrame);

		setControlButtonsPosition(&videoFrame);
		//applyFrameRotation(&videoFrame);
		while(1)
		{
			ft_int32_t parret,tempgui[2];
			ft_uint8_t pendown = 0;
			/* User touches */
			guimutex = 1;
			Ft_Gpu_Hal_RdMem(phost,REG_TOUCH_TAG_XY,tempgui,8);
			guimutex = 0;

			G_S_PGui.CurrTTag = tempgui[1];

			if(G_S_PGui.CurrTTag != 0)
				printf("Tag: %u\n");

			if(0x80000000 != (tempgui[0] & 0x80000000))
			{
				G_S_PGui.Flags |= FT_GUI_PENDOWN;
				pendown = 1;
			}
			else
			{
				G_S_PGui.Flags &= ~FT_GUI_PENDOWN;
				pendown = 0;
			}
			//pen up and previously a selection happened
			if((0 == pendown) && (0 == G_S_PGui.CurrTTag) && (0 != G_S_PGui.PrevTTag))
			{

				if(1 == G_S_PGui.PrevTTag)
				{
					G_S_PGui.TouchSel ^= 0x01;
					if(0 == (G_S_PGui.TouchSel & 0x01))
					{
						G_S_AviPlayer.StateCurr = FT_AVIPLAYER_PLAY;
#if defined(FT900_PLATFORM)
						 timer_start(timer_select_a);;
#endif
					}
					else
					{
						G_S_AviPlayer.StateCurr = FT_AVIPLAYER_PAUSE;
#if defined(FT900_PLATFORM)
						 timer_stop(timer_select_a);
#endif
						//disable the timer to simulate pause condition
					}
				}
				else if (2 == G_S_PGui.PrevTTag)
				{
#if defined(FT900_PLATFORM)
					 timer_stop(timer_select_a);
#endif
					if(videodecodemutex > 0)
					{
						ft_delay(60);//temp delay
					}
					guimutex = 1;
					Ft_Gpu_Hal_WaitCmdfifo_empty(phost);
					guimutex = 0;
					printf("\n break");
					break;
				}
				else if(3 == G_S_PGui.PrevTTag){
					if(curOrientation == FT_VIDEO_LANDSCAPE){
						curOrientation = FT_VIDEO_PORTRAIT;
						printf("Changing to portrait mode.\n");
					}
					else{
						curOrientation = FT_VIDEO_LANDSCAPE;
						printf("Changing to landscape mode.\n");
					}
				}

				G_S_PGui.Flags |= FT_GUI_REFRESH;

				//printf("gui penup %x %x %x %x %x\n",G_S_PGui.PrevTTag,G_S_PGui.CurrTTag,G_S_PGui.Flags,G_S_PGui.TouchSel,G_S_AviPlayer.StateCurr);
			}
			if((1 == pendown) && (0 != G_S_PGui.CurrTTag))
			{

				if(1 == G_S_PGui.CurrTTag)
				{
					//highlight of play/pause button
					G_S_PGui.TouchSel |= 0x02;
				}
				else if(2 == G_S_PGui.CurrTTag)
				{
					//highlight of exit button
					G_S_PGui.TouchSel |= 0x04;
				}

				G_S_PGui.Flags |= FT_GUI_REFRESH;
			}

			if(G_S_PGui.Flags & FT_GUI_PENDOWN)
			{
				G_S_PGui.Disappear = FT_GUI_DISAPPEAR_COUNT;
				if(G_S_PGui.CurrTTag == 0 && prevSelect !=0 && (prevSelect > G_S_PGui.Disappear-FT_VIDEO_TOUCH_THREASHOLD)){
					videoFrame.zoomOut ^= 1;
				}
				//frameRotationalScalingApplied=1;
			}
			else
			{
				G_S_PGui.Disappear = max(0,(G_S_PGui.Disappear - 1));
				prevSelect = G_S_PGui.Disappear;
#if defined(MSVC_FT800EMU)
				
				//if(G_S_PGui.Disappear == 0)
					//G_S_PGui.Flags &= (~FT_GUI_REFRESH);
#endif
			}


			/*
			
						if(G_S_PGui.CurrTTag == 'x'){
							if(curOrientation == FT_VIDEO_LANDSCAPE){
								curOrientation = FT_VIDEO_PORTRAIT;
								printf("Changing to portrait mode.\n");
							}
							else{
								curOrientation = FT_VIDEO_LANDSCAPE;
								printf("Changing to landscape mode.\n");
							}
						}
			*/


			/* check for the condition where single frame is greater than video ring buffer */
			/* Check the queue and decide to issue jpg decode call */
			/* check for the completion of previous decode frame */
			//if(Ft_Gpu_Hal_Rd16(phost,REG_CMD_READ) == Ft_Gpu_Hal_Rd16(phost,REG_CMD_WRITE))
			{
				/* GUI screen shot construction */
				if(G_S_PGui.Flags & FT_GUI_REFRESH)  //original
				//if(1)
				{
					guimutex = 1;
					if(Ft_Gpu_Hal_Rd16(phost,REG_CMD_READ) == Ft_Gpu_Hal_Rd16(phost,REG_CMD_WRITE))
					{
						ft_int32_t centerx,centery,totalints,profileval;
						//ft_char8_t charcurr,*pcharcurr,temparray[32];

						/* construct the DL and display */

						Ft_Gpu_CoCmd_Dlstart(phost);
						Ft_App_WrCoCmd_Buffer(phost,CLEAR_TAG(0));

						Ft_App_WrCoCmd_Buffer(phost,CLEAR_COLOR_RGB(0,0,0));
						Ft_App_WrCoCmd_Buffer(phost,CLEAR(1,1,1) );

						/*
						if(frameRotationalScalingApplied){
							calculateFrameRotatialScaling(&videoFrame);
							frameRotationalScalingApplied=0;
						}
						*/

						//applyFrameRotation(&videoFrame);
						//rotateScreen(&videoFrame, FT_VIDEO_LANDSCAPE);
						
						
						//applyFrameRotationalScaling(&videoFrame, FT_VIDEO_LANDSCAPE);
						
						


						//decoded jpeg bitmap
						Ft_App_WrCoCmd_Buffer(phost,TAG_MASK(0));
						Ft_App_WrCoCmd_Buffer(phost,TAG(0));
						Ft_App_WrCoCmd_Buffer(phost,COLOR_RGB(255,255,255));
						Ft_App_WrCoCmd_Buffer(phost,BITMAP_HANDLE(0));
						Ft_App_WrCoCmd_Buffer(phost,BITMAP_SOURCE(0));//bitmap locaiton is 4, as 0 is used for return of video decode result
						Ft_App_WrCoCmd_Buffer(phost,BITMAP_LAYOUT(RGB565,aviw*2L,avih));
						Ft_App_WrCoCmd_Buffer(phost,BITMAP_LAYOUT_H(((aviw*2L)>>10),((avih)>>9)));
						Ft_App_WrCoCmd_Buffer(phost,BITMAP_SIZE(NEAREST,BORDER,BORDER,aviw,avih));
						Ft_App_WrCoCmd_Buffer(phost,BITMAP_SIZE_H((aviw>>9),(avih>>9)));
						Ft_App_WrCoCmd_Buffer(phost,BEGIN(BITMAPS));
						//hack for not to display video frame
						if(G_SVidDataPath.NumDecoded > 1)
						{
							if(videoFrame.zoomOut){
								Ft_App_WrCoCmd_Buffer(phost,VERTEX2F(8*(FT_DispWidth - videoFrame.frameWidth),8*(FT_DispHeight - videoFrame.frameHeight)));
							}
							else{
								applyFrameRotationalScaling(&videoFrame, curOrientation);
							//Ft_App_WrCoCmd_Buffer(phost,VERTEX2F(8*(FT_DispWidth - finalFrameWidth),8*(FT_DispHeight - finalFrameHeight)));//display at the center of the screen
								Ft_App_WrCoCmd_Buffer(phost,VERTEX2F(8*(FT_DispWidth - videoFrame.finalFrameWidth),8*(FT_DispHeight - videoFrame.finalFrameHeight)));
							//Ft_App_WrCoCmd_Buffer(phost,VERTEX2F(8*(FT_DispWidth - videoFrame.frameWidth),8*(FT_DispHeight - videoFrame.frameHeight)));
							}
						}
						
						//restoreFrameScaling();
						//restoreFrameRotation();
						//Ft_Gpu_CoCmd_SetRotate(phost, 0);

						//drawControlButtons(&controlButtons);

						if(G_S_PGui.Disappear > 0)
						{
							
							drawControlButtons(&controlButtons);
#if 0
						//draw three rectangles, 1 triangle, 2 lines,
						Ft_App_WrCoCmd_Buffer(phost,TAG_MASK(1));

						Ft_App_WrCoCmd_Buffer(phost,COLOR_RGB(32,32,32));
						Ft_App_WrCoCmd_Buffer(phost,COLOR_A(128));
						Ft_App_WrCoCmd_Buffer(phost,LINE_WIDTH(1*16));
						Ft_App_WrCoCmd_Buffer(phost,BEGIN(RECTS));

						//center rectangle
#if defined(DISPLAY_RESOLUTION_WVGA)
						centerx = 400;
						centery = 400;
#elif defined(DISPLAY_RESOLUTION_HVGA_PORTRAITl)
						centerx = 135;
						centery = 417;
#else
						centerx = 400;
						centery = 400;
#endif
						Ft_App_WrCoCmd_Buffer(phost,VERTEX2F(16*(centerx - 240),16*(centery - 25)));
						Ft_App_WrCoCmd_Buffer(phost,VERTEX2F(16*(centerx + 240),16*(centery + 25)));

						//play/pause rectangle
						#if defined(DISPLAY_RESOLUTION_WVGA)
						centerx = 131;
						centery = 400;
						#elif defined(DISPLAY_RESOLUTION_HVGA_PORTRAITl)
						centerx = 33;
						centery = 417;
						#else
						//centerx = 131;
						//centery = 400;
						centerx = 20;
						centery = 20;
						#endif
						Ft_App_WrCoCmd_Buffer(phost,TAG(1));
						Ft_App_WrCoCmd_Buffer(phost,VERTEX2F(16*(centerx - 25),16*(centery - 25)));
						Ft_App_WrCoCmd_Buffer(phost,VERTEX2F(16*(centerx + 25),16*(centery + 25)));

						if(G_S_PGui.TouchSel & 0x02)
						{
							Ft_App_WrCoCmd_Buffer(phost,COLOR_A(255));
							Ft_App_WrCoCmd_Buffer(phost,VERTEX2F(16*(centerx - 23),16*(centery - 23)));
							Ft_App_WrCoCmd_Buffer(phost,VERTEX2F(16*(centerx + 23),16*(centery + 23)));
							Ft_App_WrCoCmd_Buffer(phost,COLOR_A(128));
						}

						//exit rectangle
						#if defined(DISPLAY_RESOLUTION_WVGA)
						centerx = 669;
						centery = 400;
						#elif defined(DISPLAY_RESOLUTION_HVGA_PORTRAITl)
						centerx = 290;
						centery = 417;		
						#else
						centerx = 669;
						centery = 400;		
						#endif
						
						Ft_App_WrCoCmd_Buffer(phost,TAG(2));
						Ft_App_WrCoCmd_Buffer(phost,VERTEX2F(16*(centerx - 25),16*(centery - 25)));
						Ft_App_WrCoCmd_Buffer(phost,VERTEX2F(16*(centerx + 25),16*(centery + 25)));

						Ft_App_WrCoCmd_Buffer(phost,COLOR_A(255));
						if(G_S_PGui.TouchSel & 0x04)
						{
							Ft_App_WrCoCmd_Buffer(phost,COLOR_A(255));
							Ft_App_WrCoCmd_Buffer(phost,VERTEX2F(16*(centerx - 23),16*(centery - 23)));
							Ft_App_WrCoCmd_Buffer(phost,VERTEX2F(16*(centerx + 23),16*(centery + 23)));
						}

						//triangle for play and two lines for pause
						Ft_App_WrCoCmd_Buffer(phost,TAG(1));
						#if defined(DISPLAY_RESOLUTION_WVGA)
						centerx = 131;
						centery = 400;
						#elif defined(DISPLAY_RESOLUTION_HVGA_PORTRAITl)
						centerx = 33;
						centery = 417;
						#else
						//centerx = 131;
						//centery = 400;
						centerx = 20;
						centery = 20;
						#endif
						Ft_App_WrCoCmd_Buffer(phost,COLOR_RGB(255,255,255));
						if(0x00 == (G_S_PGui.TouchSel & 0x01))
						{
							//pause button
							Ft_App_WrCoCmd_Buffer(phost,LINE_WIDTH(4*16));
							Ft_App_WrCoCmd_Buffer(phost,BEGIN(LINES));

							Ft_App_WrCoCmd_Buffer(phost,VERTEX2F(16*(centerx - 8),16*(centery - 15)));
							Ft_App_WrCoCmd_Buffer(phost,VERTEX2F(16*(centerx - 8),16*(centery + 15)));

							Ft_App_WrCoCmd_Buffer(phost,VERTEX2F(16*(centerx + 8),16*(centery - 15)));
							Ft_App_WrCoCmd_Buffer(phost,VERTEX2F(16*(centerx + 8),16*(centery + 15)));
						}
						else
						{
							//play button
							Ft_App_WrCoCmd_Buffer(phost,LINE_WIDTH(1*16));
							Ft_App_WrCoCmd_Buffer(phost,BEGIN(EDGE_STRIP_L));
							Ft_App_WrCoCmd_Buffer(phost,SCISSOR_XY(centerx - 25,centery - 25));
							Ft_App_WrCoCmd_Buffer(phost,SCISSOR_SIZE(50,50));

							Ft_App_WrCoCmd_Buffer(phost,COLOR_MASK(0,0,0,1));
							Ft_App_WrCoCmd_Buffer(phost,CLEAR(1,0,0));
							Ft_App_WrCoCmd_Buffer(phost,STENCIL_OP(INCR,INCR));

							Ft_App_WrCoCmd_Buffer(phost,VERTEX2F(16*(centerx + 15),16*(centery)));
							Ft_App_WrCoCmd_Buffer(phost,VERTEX2F(16*(centerx - 8),16*(centery - 15)));
							Ft_App_WrCoCmd_Buffer(phost,VERTEX2F(16*(centerx - 8),16*(centery + 15)));
							Ft_App_WrCoCmd_Buffer(phost,VERTEX2F(16*(centerx + 15),16*(centery)));
							Ft_App_WrCoCmd_Buffer(phost,END());
							Ft_App_WrCoCmd_Buffer(phost,BEGIN(EDGE_STRIP_L));
							Ft_App_WrCoCmd_Buffer(phost,STENCIL_FUNC(EQUAL,1,255));
							Ft_App_WrCoCmd_Buffer(phost,COLOR_MASK(1,1,1,1));
							Ft_App_WrCoCmd_Buffer(phost,VERTEX2F(16*(centerx + 25),16*(centery - 25)));
							Ft_App_WrCoCmd_Buffer(phost,VERTEX2F(16*(centerx + 25),16*(centery + 25)));

							Ft_App_WrCoCmd_Buffer(phost,STENCIL_FUNC(ALWAYS,1,255));
							Ft_App_WrCoCmd_Buffer(phost,BEGIN(LINE_STRIP));
							Ft_App_WrCoCmd_Buffer(phost,VERTEX2F(16*(centerx + 15),16*(centery)));
							Ft_App_WrCoCmd_Buffer(phost,VERTEX2F(16*(centerx - 8),16*(centery - 15)));
							Ft_App_WrCoCmd_Buffer(phost,VERTEX2F(16*(centerx - 8),16*(centery + 15)));
							Ft_App_WrCoCmd_Buffer(phost,VERTEX2F(16*(centerx + 15),16*(centery)));
							Ft_App_WrCoCmd_Buffer(phost,SCISSOR_XY(0,0));
							Ft_App_WrCoCmd_Buffer(phost,SCISSOR_SIZE(2048,2048));

						}

						//x for exit button
						Ft_App_WrCoCmd_Buffer(phost,TAG(2));
						Ft_App_WrCoCmd_Buffer(phost,LINE_WIDTH(2*16));
						Ft_App_WrCoCmd_Buffer(phost,BEGIN(LINES));
						centerx = 669;
						centery = 400;


						Ft_App_WrCoCmd_Buffer(phost,VERTEX2F(16*(centerx - 9),16*(centery - 15)));
						Ft_App_WrCoCmd_Buffer(phost,VERTEX2F(16*(centerx + 9),16*(centery + 15)));
						Ft_App_WrCoCmd_Buffer(phost,VERTEX2F(16*(centerx + 9),16*(centery - 15)));
						Ft_App_WrCoCmd_Buffer(phost,VERTEX2F(16*(centerx - 9),16*(centery + 15)));

						//status bar
						centerx = 400;
						centery = 400;
						Ft_App_WrCoCmd_Buffer(phost,TAG_MASK(0));
						Ft_App_WrCoCmd_Buffer(phost,LINE_WIDTH(8*16));
						Ft_App_WrCoCmd_Buffer(phost,COLOR_RGB(0,0,64));
						Ft_App_WrCoCmd_Buffer(phost,VERTEX2F(16*(centerx - 190),16*(centery)));
						Ft_App_WrCoCmd_Buffer(phost,VERTEX2F(16*(centerx + 190),16*(centery)));
						Ft_App_WrCoCmd_Buffer(phost,COLOR_RGB(0,0,192));
						Ft_App_WrCoCmd_Buffer(phost,VERTEX2F(16*(centerx - 190),16*(centery)));

						profileval = (G_S_AviParser.SVid.CurrFrame*380)/G_S_AviParser.SVid.TotFrames;
						Ft_App_WrCoCmd_Buffer(phost,VERTEX2F(16*(centerx - 190 + profileval),16*(centery)));

						Ft_App_WrCoCmd_Buffer(phost,TAG_MASK(1));
						

#endif					
						}
						/*
						Ft_App_WrCoCmd_Buffer(phost,TAG_MASK(1));
						Ft_App_WrCoCmd_Buffer(phost,COLOR_A(255));
						Ft_App_WrCoCmd_Buffer(phost,COLOR_RGB(255,255,255));
						Ft_App_WrCoCmd_Buffer(phost,BEGIN(FTPOINTS));
						Ft_App_WrCoCmd_Buffer(phost,POINT_SIZE(20*16));
						Ft_App_WrCoCmd_Buffer(phost,TAG('x'));
						Ft_App_WrCoCmd_Buffer(phost,VERTEX2F((FT_DispWidth/2)*16,(FT_DispHeight/2)*16));
						*/

						Ft_App_WrCoCmd_Buffer(phost,DISPLAY());
						Ft_Gpu_CoCmd_Swap(phost);
						Ft_App_Flush_Co_Buffer(phost);
						//Ft_Gpu_Hal_WaitCmdfifo_empty(phost);



						G_S_PGui.Flags &= (~FT_GUI_REFRESH);//default the refresh
					}
					guimutex = 0;

				}
			}

			parret = 0;
			/* GUI handling based on fps, check for user selction and perform operation accordingly */

#if defined(MSVC_PLATFORM) || defined(MSVC_FT800EMU)
			///TODO: Windows logic
#elif defined(FT900_PLATFORM)
			if((G_S_AviParser.SVid.CurrFrame) < (ft_timer_interrupt1_get()))   ///ft_timer_interrupt1_get() seems it's returning the current amount of time ticks
			{
				parret = 1;
			}
#endif

#if defined(FT900_PLATFORM)
			//if(1 == parret)
			if(videodecodemutex > 0)
			{
				guimutex = 1;
#endif
			if(Ft_Gpu_Hal_Rd16(phost,REG_CMD_READ) == Ft_Gpu_Hal_Rd16(phost,REG_CMD_WRITE))
			{
				if(srqvidin.ValidEle >= 1)
				{
					ft_uint8_t valideles = srqvidin.ValidEle;
					FT_MetaV_t tempvmeta;

					/* Read the element and trigger a decode */
					//printf("ft81x status bfr %x %x %x %x \n",Ft_Gpu_Hal_Rd32(phost,REG_CMD_READ),Ft_Gpu_Hal_Rd32(phost,REG_CMD_WRITE),Ft_Gpu_Hal_Rd32(phost,REG_MEDIAFIFO_READ),Ft_Gpu_Hal_Rd32(phost,REG_MEDIAFIFO_WRITE));
					FT_RQueue_PeekE(&srqvidin,&tempvmeta,srqvidin.ReadIdx);
					if(0 == tempvmeta.PartialSz)
					{
						FT_RQueue_ReadE(&srqvidin,&tempvmeta);
						if(tempvmeta.Length > 0)
						{
							Ft_Gpu_CoCmd_LoadImage(phost,0, OPT_MEDIAFIFO | OPT_NODL);
							Ft_App_Flush_Co_Buffer(phost);
							G_SVidDataPath.NumDecoded++;
						}
						G_S_AviPlayer.TotalBytes += tempvmeta.Length;//total number of bytes consumed till now by decoder

						videodecodemutex--;//hack
					}

				}

			}
#if defined(FT900_PLATFORM)
			guimutex = 0;
			}
#endif
			parserfillmutex = 1;
			parret = FT_AVIParser_FillChunk(&G_S_AviParser,&G_SVidDataPath,&G_SAudDataPath);
			parserfillmutex = 0;


#if defined(FT900_PLATFORM)
        	if(0 == (FTPayer_TMicroS % (30*10)))
        	{
        		printf("profile for every 300 ticks %d %d %d %d %d %d %x\n",FTPayer_TMicroS,G_S_AviParser.SVid.CurrFrame,G_S_AviPlayer.TotalBytes,G_SVidDataPath.NumDecoded,G_S_PGui.PrevTTag,G_S_PGui.TouchSel,videodecodemutex);
        	}

        	if(0 == (FTPayer_TMicroS % (G_S_AviParser.SVid.DataRate)))
        	{
        		G_S_PGui.Flags |= (FT_GUI_REFRESH);
        		//printf("Swap enable %x %d %d %d\n",G_S_PGui.Flags,FTPayer_TMicroS,G_SVidDataPath.NumDecoded,G_S_AviParser.SVid.CurrFrame);
        	}
#endif

			if(FT_AVIPARSER_EOF == parret)
			{

				parserfillmutex = 1;
				Ft_Gpu_Hal_WaitCmdfifo_empty(phost);
				parserfillmutex = 0;

				break;
			}
			else if(parret != FT_AVIPARSER_OK)
			{

			}

			//printf("ft81x status %x %x %x %x \n",Ft_Gpu_Hal_Rd32(phost,REG_CMD_READ),Ft_Gpu_Hal_Rd32(phost,REG_CMD_WRITE),Ft_Gpu_Hal_Rd32(phost,REG_MEDIAFIFO_READ),Ft_Gpu_Hal_Rd32(phost,REG_MEDIAFIFO_WRITE));
			if(srqaudin.ValidEle >= 2)
			{
				ft_uint8_t valideles = srqaudin.ValidEle;
				FT_MetaA_t tempameta;
				//printf("Player flush audq %d\n",srqaudin.ValidEle);
				for(i=0;i<valideles - 1;i++)
				{
					FT_RQueue_ReadE(&srqaudin,&tempameta);
				}
			}

			G_S_PGui.PrevTTag = G_S_PGui.CurrTTag;
			G_S_PGui.TouchSel &= (~ 0x06);
		}

		/* exit avi parser */
		printf("\n exit");
		FT_AVIParser_Exit(&G_S_AviParser);
		//restoreFrameRotationalScaling(&videoFrame);
		k++;
		if(k >= FT_PLAYER_TESTNUM)
		{
			k = 0;
		}

		if((2 == G_S_PGui.PrevTTag))
		{
			break;
		}
	}
	}
	/* Close video data path */
	printf("\n close");
	Ft_Gpu_Hal_Close(phost);
	Ft_Gpu_Hal_DeInit();



#if defined(MSVC_PLATFORM) || defined(MSVC_FT800EMU)
	return 0;
#endif


}


/* meta data related helper apis */
ft_int32_t FT_MetaV_Init(FT_MetaV_t *pMV)
{
	pMV->Pts = 0;		
	pMV->Curr = 0;		
	pMV->Length = 0;		
	pMV->PartialSz = 0;	
	pMV->Flags = FT_AVIPARSER_FLAGSNONE;
	
	return 0;
}
ft_int32_t FT_MetaA_Init(FT_MetaA_t *pMA)
{
	pMA->Pts = 0;		
	pMA->Curr = 0;		
	pMA->Length = 0;		
	pMA->PartialSz = 0;	
	pMA->Flags = FT_AVIPARSER_FLAGSNONE;
	
	return 0;
}
ft_int32_t FT_AVIRiffE_Init(FT_AVIRiff_t *pare)
{
	/* memset could be utilized */
	pare->RiffOff = 0;
	pare->RiffSize = 0;
	pare->MoviOff = 0;
	pare->MoviSize = 0;
	pare->indxOff = 0;
	pare->indxSize = 0;
	
	return 0;
}



void loop()
{
}

DWORD get_fattime (void)
{
	/* Returns current time packed into a DWORD variable */
	return 0;
}

/* Nothing beyond this */













